/****************************************************************************
** Meta object code from reading C++ file 'device.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/device.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'device.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14NetworkManager6DeviceE_t {};
} // unnamed namespace

template <> constexpr inline auto NetworkManager::Device::qt_create_metaobjectdata<qt_meta_tag_ZN14NetworkManager6DeviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NetworkManager::Device",
        "stateChanged",
        "",
        "NetworkManager::Device::State",
        "newstate",
        "oldstate",
        "NetworkManager::Device::StateChangeReason",
        "reason",
        "activeConnectionChanged",
        "autoconnectChanged",
        "availableConnectionChanged",
        "availableConnectionAppeared",
        "connection",
        "availableConnectionDisappeared",
        "capabilitiesChanged",
        "dhcp4ConfigChanged",
        "dhcp6ConfigChanged",
        "driverChanged",
        "driverVersionChanged",
        "firmwareMissingChanged",
        "firmwareVersionChanged",
        "interfaceNameChanged",
        "ipV4AddressChanged",
        "ipV4ConfigChanged",
        "ipV6ConfigChanged",
        "ipInterfaceChanged",
        "managedChanged",
        "interfaceFlagsChanged",
        "physicalPortIdChanged",
        "mtuChanged",
        "nmPluginMissingChanged",
        "nmPluginMissing",
        "meteredChanged",
        "MeteredStatus",
        "metered",
        "connectionStateChanged",
        "stateReasonChanged",
        "udiChanged",
        "uni",
        "interfaceName",
        "ipInterfaceName",
        "driver",
        "driverVersion",
        "firmwareVersion",
        "genericCapabilities",
        "QVariant",
        "ipV4Address",
        "QHostAddress",
        "managed",
        "mtu",
        "InterfaceFlags",
        "Interfaceflags",
        "udi",
        "firmwareMissing",
        "autoconnect",
        "stateReason",
        "DeviceStateReason",
        "state",
        "State",
        "deviceStatistics",
        "NetworkManager::DeviceStatistics::Ptr",
        "UnknownState",
        "Unmanaged",
        "Unavailable",
        "Disconnected",
        "Preparing",
        "ConfiguringHardware",
        "NeedAuth",
        "ConfiguringIp",
        "CheckingIp",
        "WaitingForSecondaries",
        "Activated",
        "Deactivating",
        "Failed",
        "StateChangeReason",
        "UnknownReason",
        "NoReason",
        "NowManagedReason",
        "NowUnmanagedReason",
        "ConfigFailedReason",
        "ConfigUnavailableReason",
        "ConfigExpiredReason",
        "NoSecretsReason",
        "AuthSupplicantDisconnectReason",
        "AuthSupplicantConfigFailedReason",
        "AuthSupplicantFailedReason",
        "AuthSupplicantTimeoutReason",
        "PppStartFailedReason",
        "PppDisconnectReason",
        "PppFailedReason",
        "DhcpStartFailedReason",
        "DhcpErrorReason",
        "DhcpFailedReason",
        "SharedStartFailedReason",
        "SharedFailedReason",
        "AutoIpStartFailedReason",
        "AutoIpErrorReason",
        "AutoIpFailedReason",
        "ModemBusyReason",
        "ModemNoDialToneReason",
        "ModemNoCarrierReason",
        "ModemDialTimeoutReason",
        "ModemDialFailedReason",
        "ModemInitFailedReason",
        "GsmApnSelectFailedReason",
        "GsmNotSearchingReason",
        "GsmRegistrationDeniedReason",
        "GsmRegistrationTimeoutReason",
        "GsmRegistrationFailedReason",
        "GsmPinCheckFailedReason",
        "FirmwareMissingReason",
        "DeviceRemovedReason",
        "SleepingReason",
        "ConnectionRemovedReason",
        "UserRequestedReason",
        "CarrierReason",
        "ConnectionAssumedReason",
        "SupplicantAvailableReason",
        "ModemNotFoundReason",
        "BluetoothFailedReason",
        "GsmSimNotInserted",
        "GsmSimPinRequired",
        "GsmSimPukRequired",
        "GsmSimWrong",
        "InfiniBandMode",
        "DependencyFailed",
        "Br2684Failed",
        "ModemManagerUnavailable",
        "SsidNotFound",
        "SecondaryConnectionFailed",
        "DcbFcoeFailed",
        "TeamdControlFailed",
        "ModemFailed",
        "ModemAvailable",
        "SimPinIncorrect",
        "NewActivation",
        "ParentChanged",
        "ParentManagedChanged",
        "Reserved",
        "UnknownStatus",
        "Yes",
        "No",
        "GuessYes",
        "GuessNo",
        "Capability",
        "IsManageable",
        "SupportsCarrierDetect",
        "Capabilities",
        "Interfaceflag",
        "None",
        "Up",
        "LowerUp",
        "Carrier",
        "Type",
        "UnknownType",
        "Ethernet",
        "Wifi",
        "Unused1",
        "Unused2",
        "Bluetooth",
        "OlpcMesh",
        "Wimax",
        "Modem",
        "InfiniBand",
        "Bond",
        "Vlan",
        "Adsl",
        "Bridge",
        "Generic",
        "Team",
        "Gre",
        "MacVlan",
        "Tun",
        "Veth",
        "IpTunnel",
        "VxLan",
        "MacSec",
        "Dummy",
        "Ppp",
        "OvsInterface",
        "OvsPort",
        "OvsBridge",
        "Wpan",
        "Lowpan",
        "WireGuard",
        "WifiP2P",
        "VRF",
        "Loopback",
        "Types"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(NetworkManager::Device::State, NetworkManager::Device::State, NetworkManager::Device::StateChangeReason)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'activeConnectionChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'autoconnectChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'availableConnectionChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'availableConnectionAppeared'
        QtMocHelpers::SignalData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Signal 'availableConnectionDisappeared'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Signal 'capabilitiesChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dhcp4ConfigChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dhcp6ConfigChanged'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'driverChanged'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'driverVersionChanged'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firmwareMissingChanged'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firmwareVersionChanged'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'interfaceNameChanged'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ipV4AddressChanged'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ipV4ConfigChanged'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ipV6ConfigChanged'
        QtMocHelpers::SignalData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ipInterfaceChanged'
        QtMocHelpers::SignalData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'managedChanged'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'interfaceFlagsChanged'
        QtMocHelpers::SignalData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'physicalPortIdChanged'
        QtMocHelpers::SignalData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mtuChanged'
        QtMocHelpers::SignalData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'nmPluginMissingChanged'
        QtMocHelpers::SignalData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Signal 'meteredChanged'
        QtMocHelpers::SignalData<void(enum MeteredStatus)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Signal 'connectionStateChanged'
        QtMocHelpers::SignalData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stateReasonChanged'
        QtMocHelpers::SignalData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'udiChanged'
        QtMocHelpers::SignalData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'uni'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'interfaceName'
        QtMocHelpers::PropertyData<QString>(39, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'ipInterfaceName'
        QtMocHelpers::PropertyData<QString>(40, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'driver'
        QtMocHelpers::PropertyData<QString>(41, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'driverVersion'
        QtMocHelpers::PropertyData<QString>(42, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'firmwareVersion'
        QtMocHelpers::PropertyData<QString>(43, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'genericCapabilities'
        QtMocHelpers::PropertyData<QVariant>(44, 0x80000000 | 45, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'ipV4Address'
        QtMocHelpers::PropertyData<QHostAddress>(46, 0x80000000 | 47, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'managed'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'mtu'
        QtMocHelpers::PropertyData<uint>(49, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'InterfaceFlags'
        QtMocHelpers::PropertyData<Interfaceflags>(50, 0x80000000 | 51, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'nmPluginMissing'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'metered'
        QtMocHelpers::PropertyData<enum MeteredStatus>(34, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'udi'
        QtMocHelpers::PropertyData<QString>(52, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'firmwareMissing'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'autoconnect'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'stateReason'
        QtMocHelpers::PropertyData<DeviceStateReason>(55, 0x80000000 | 56, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'state'
        QtMocHelpers::PropertyData<enum State>(57, 0x80000000 | 58, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'deviceStatistics'
        QtMocHelpers::PropertyData<NetworkManager::DeviceStatistics::Ptr>(59, 0x80000000 | 60, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(58, 58, QMC::EnumFlags{}).add({
            {   61, State::UnknownState },
            {   62, State::Unmanaged },
            {   63, State::Unavailable },
            {   64, State::Disconnected },
            {   65, State::Preparing },
            {   66, State::ConfiguringHardware },
            {   67, State::NeedAuth },
            {   68, State::ConfiguringIp },
            {   69, State::CheckingIp },
            {   70, State::WaitingForSecondaries },
            {   71, State::Activated },
            {   72, State::Deactivating },
            {   73, State::Failed },
        }),
        // enum 'StateChangeReason'
        QtMocHelpers::EnumData<enum StateChangeReason>(74, 74, QMC::EnumFlags{}).add({
            {   75, StateChangeReason::UnknownReason },
            {   76, StateChangeReason::NoReason },
            {   77, StateChangeReason::NowManagedReason },
            {   78, StateChangeReason::NowUnmanagedReason },
            {   79, StateChangeReason::ConfigFailedReason },
            {   80, StateChangeReason::ConfigUnavailableReason },
            {   81, StateChangeReason::ConfigExpiredReason },
            {   82, StateChangeReason::NoSecretsReason },
            {   83, StateChangeReason::AuthSupplicantDisconnectReason },
            {   84, StateChangeReason::AuthSupplicantConfigFailedReason },
            {   85, StateChangeReason::AuthSupplicantFailedReason },
            {   86, StateChangeReason::AuthSupplicantTimeoutReason },
            {   87, StateChangeReason::PppStartFailedReason },
            {   88, StateChangeReason::PppDisconnectReason },
            {   89, StateChangeReason::PppFailedReason },
            {   90, StateChangeReason::DhcpStartFailedReason },
            {   91, StateChangeReason::DhcpErrorReason },
            {   92, StateChangeReason::DhcpFailedReason },
            {   93, StateChangeReason::SharedStartFailedReason },
            {   94, StateChangeReason::SharedFailedReason },
            {   95, StateChangeReason::AutoIpStartFailedReason },
            {   96, StateChangeReason::AutoIpErrorReason },
            {   97, StateChangeReason::AutoIpFailedReason },
            {   98, StateChangeReason::ModemBusyReason },
            {   99, StateChangeReason::ModemNoDialToneReason },
            {  100, StateChangeReason::ModemNoCarrierReason },
            {  101, StateChangeReason::ModemDialTimeoutReason },
            {  102, StateChangeReason::ModemDialFailedReason },
            {  103, StateChangeReason::ModemInitFailedReason },
            {  104, StateChangeReason::GsmApnSelectFailedReason },
            {  105, StateChangeReason::GsmNotSearchingReason },
            {  106, StateChangeReason::GsmRegistrationDeniedReason },
            {  107, StateChangeReason::GsmRegistrationTimeoutReason },
            {  108, StateChangeReason::GsmRegistrationFailedReason },
            {  109, StateChangeReason::GsmPinCheckFailedReason },
            {  110, StateChangeReason::FirmwareMissingReason },
            {  111, StateChangeReason::DeviceRemovedReason },
            {  112, StateChangeReason::SleepingReason },
            {  113, StateChangeReason::ConnectionRemovedReason },
            {  114, StateChangeReason::UserRequestedReason },
            {  115, StateChangeReason::CarrierReason },
            {  116, StateChangeReason::ConnectionAssumedReason },
            {  117, StateChangeReason::SupplicantAvailableReason },
            {  118, StateChangeReason::ModemNotFoundReason },
            {  119, StateChangeReason::BluetoothFailedReason },
            {  120, StateChangeReason::GsmSimNotInserted },
            {  121, StateChangeReason::GsmSimPinRequired },
            {  122, StateChangeReason::GsmSimPukRequired },
            {  123, StateChangeReason::GsmSimWrong },
            {  124, StateChangeReason::InfiniBandMode },
            {  125, StateChangeReason::DependencyFailed },
            {  126, StateChangeReason::Br2684Failed },
            {  127, StateChangeReason::ModemManagerUnavailable },
            {  128, StateChangeReason::SsidNotFound },
            {  129, StateChangeReason::SecondaryConnectionFailed },
            {  130, StateChangeReason::DcbFcoeFailed },
            {  131, StateChangeReason::TeamdControlFailed },
            {  132, StateChangeReason::ModemFailed },
            {  133, StateChangeReason::ModemAvailable },
            {  134, StateChangeReason::SimPinIncorrect },
            {  135, StateChangeReason::NewActivation },
            {  136, StateChangeReason::ParentChanged },
            {  137, StateChangeReason::ParentManagedChanged },
            {  138, StateChangeReason::Reserved },
        }),
        // enum 'MeteredStatus'
        QtMocHelpers::EnumData<enum MeteredStatus>(33, 33, QMC::EnumFlags{}).add({
            {  139, MeteredStatus::UnknownStatus },
            {  140, MeteredStatus::Yes },
            {  141, MeteredStatus::No },
            {  142, MeteredStatus::GuessYes },
            {  143, MeteredStatus::GuessNo },
        }),
        // enum 'Capability'
        QtMocHelpers::EnumData<enum Capability>(144, 144, QMC::EnumFlags{}).add({
            {  145, Capability::IsManageable },
            {  146, Capability::SupportsCarrierDetect },
        }),
        // flag 'Capabilities'
        QtMocHelpers::EnumData<Capabilities>(147, 144, QMC::EnumIsFlag).add({
            {  145, Capability::IsManageable },
            {  146, Capability::SupportsCarrierDetect },
        }),
        // enum 'Interfaceflag'
        QtMocHelpers::EnumData<enum Interfaceflag>(148, 148, QMC::EnumFlags{}).add({
            {  149, Interfaceflag::None },
            {  150, Interfaceflag::Up },
            {  151, Interfaceflag::LowerUp },
            {  152, Interfaceflag::Carrier },
        }),
        // flag 'Interfaceflags'
        QtMocHelpers::EnumData<Interfaceflags>(51, 148, QMC::EnumIsFlag).add({
            {  149, Interfaceflag::None },
            {  150, Interfaceflag::Up },
            {  151, Interfaceflag::LowerUp },
            {  152, Interfaceflag::Carrier },
        }),
        // enum 'Type'
        QtMocHelpers::EnumData<enum Type>(153, 153, QMC::EnumFlags{}).add({
            {  154, Type::UnknownType },
            {  155, Type::Ethernet },
            {  156, Type::Wifi },
            {  157, Type::Unused1 },
            {  158, Type::Unused2 },
            {  159, Type::Bluetooth },
            {  160, Type::OlpcMesh },
            {  161, Type::Wimax },
            {  162, Type::Modem },
            {  163, Type::InfiniBand },
            {  164, Type::Bond },
            {  165, Type::Vlan },
            {  166, Type::Adsl },
            {  167, Type::Bridge },
            {  168, Type::Generic },
            {  169, Type::Team },
            {  170, Type::Gre },
            {  171, Type::MacVlan },
            {  172, Type::Tun },
            {  173, Type::Veth },
            {  174, Type::IpTunnel },
            {  175, Type::VxLan },
            {  176, Type::MacSec },
            {  177, Type::Dummy },
            {  178, Type::Ppp },
            {  179, Type::OvsInterface },
            {  180, Type::OvsPort },
            {  181, Type::OvsBridge },
            {  182, Type::Wpan },
            {  183, Type::Lowpan },
            {  184, Type::WireGuard },
            {  185, Type::WifiP2P },
            {  186, Type::VRF },
            {  187, Type::Loopback },
        }),
        // flag 'Types'
        QtMocHelpers::EnumData<Types>(188, 153, QMC::EnumIsFlag).add({
            {  154, Type::UnknownType },
            {  155, Type::Ethernet },
            {  156, Type::Wifi },
            {  157, Type::Unused1 },
            {  158, Type::Unused2 },
            {  159, Type::Bluetooth },
            {  160, Type::OlpcMesh },
            {  161, Type::Wimax },
            {  162, Type::Modem },
            {  163, Type::InfiniBand },
            {  164, Type::Bond },
            {  165, Type::Vlan },
            {  166, Type::Adsl },
            {  167, Type::Bridge },
            {  168, Type::Generic },
            {  169, Type::Team },
            {  170, Type::Gre },
            {  171, Type::MacVlan },
            {  172, Type::Tun },
            {  173, Type::Veth },
            {  174, Type::IpTunnel },
            {  175, Type::VxLan },
            {  176, Type::MacSec },
            {  177, Type::Dummy },
            {  178, Type::Ppp },
            {  179, Type::OvsInterface },
            {  180, Type::OvsPort },
            {  181, Type::OvsBridge },
            {  182, Type::Wpan },
            {  183, Type::Lowpan },
            {  184, Type::WireGuard },
            {  185, Type::WifiP2P },
            {  186, Type::VRF },
            {  187, Type::Loopback },
        }),
    };
    return QtMocHelpers::metaObjectData<Device, qt_meta_tag_ZN14NetworkManager6DeviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN14NetworkManager6DeviceE[] = {
    QMetaObject::SuperData::link<NetworkManager::DeviceStatistics::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject NetworkManager::Device::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager6DeviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager6DeviceE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN14NetworkManager6DeviceE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14NetworkManager6DeviceE_t>.metaTypes,
    nullptr
} };

void NetworkManager::Device::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Device *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<NetworkManager::Device::State>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<NetworkManager::Device::State>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<NetworkManager::Device::StateChangeReason>>(_a[3]))); break;
        case 1: _t->activeConnectionChanged(); break;
        case 2: _t->autoconnectChanged(); break;
        case 3: _t->availableConnectionChanged(); break;
        case 4: _t->availableConnectionAppeared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->availableConnectionDisappeared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->capabilitiesChanged(); break;
        case 7: _t->dhcp4ConfigChanged(); break;
        case 8: _t->dhcp6ConfigChanged(); break;
        case 9: _t->driverChanged(); break;
        case 10: _t->driverVersionChanged(); break;
        case 11: _t->firmwareMissingChanged(); break;
        case 12: _t->firmwareVersionChanged(); break;
        case 13: _t->interfaceNameChanged(); break;
        case 14: _t->ipV4AddressChanged(); break;
        case 15: _t->ipV4ConfigChanged(); break;
        case 16: _t->ipV6ConfigChanged(); break;
        case 17: _t->ipInterfaceChanged(); break;
        case 18: _t->managedChanged(); break;
        case 19: _t->interfaceFlagsChanged(); break;
        case 20: _t->physicalPortIdChanged(); break;
        case 21: _t->mtuChanged(); break;
        case 22: _t->nmPluginMissingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->meteredChanged((*reinterpret_cast<std::add_pointer_t<enum MeteredStatus>>(_a[1]))); break;
        case 24: _t->connectionStateChanged(); break;
        case 25: _t->stateReasonChanged(); break;
        case 26: _t->udiChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Device::*)(NetworkManager::Device::State , NetworkManager::Device::State , NetworkManager::Device::StateChangeReason )>(_a, &Device::stateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::activeConnectionChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::autoconnectChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::availableConnectionChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)(const QString & )>(_a, &Device::availableConnectionAppeared, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)(const QString & )>(_a, &Device::availableConnectionDisappeared, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::capabilitiesChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::dhcp4ConfigChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::dhcp6ConfigChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::driverChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::driverVersionChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::firmwareMissingChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::firmwareVersionChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::interfaceNameChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::ipV4AddressChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::ipV4ConfigChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::ipV6ConfigChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::ipInterfaceChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::managedChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::interfaceFlagsChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::physicalPortIdChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::mtuChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)(bool )>(_a, &Device::nmPluginMissingChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)(MeteredStatus )>(_a, &Device::meteredChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::connectionStateChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::stateReasonChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::udiChanged, 26))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->uni(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->interfaceName(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->ipInterfaceName(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->driver(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->driverVersion(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->firmwareVersion(); break;
        case 6: *reinterpret_cast<QVariant*>(_v) = _t->capabilitiesV(); break;
        case 7: *reinterpret_cast<QHostAddress*>(_v) = _t->ipV4Address(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->managed(); break;
        case 9: *reinterpret_cast<uint*>(_v) = _t->mtu(); break;
        case 10: QtMocHelpers::assignFlags<Interfaceflags>(_v, _t->interfaceFlags()); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->nmPluginMissing(); break;
        case 12: *reinterpret_cast<enum MeteredStatus*>(_v) = _t->metered(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->udi(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->firmwareMissing(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->autoconnect(); break;
        case 16: *reinterpret_cast<DeviceStateReason*>(_v) = _t->stateReason(); break;
        case 17: *reinterpret_cast<enum State*>(_v) = _t->state(); break;
        case 18: *reinterpret_cast<NetworkManager::DeviceStatistics::Ptr*>(_v) = _t->deviceStatistics(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 8: _t->setManaged(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setAutoconnect(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *NetworkManager::Device::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NetworkManager::Device::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager6DeviceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int NetworkManager::Device::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 27;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void NetworkManager::Device::stateChanged(NetworkManager::Device::State _t1, NetworkManager::Device::State _t2, NetworkManager::Device::StateChangeReason _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void NetworkManager::Device::activeConnectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void NetworkManager::Device::autoconnectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void NetworkManager::Device::availableConnectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void NetworkManager::Device::availableConnectionAppeared(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void NetworkManager::Device::availableConnectionDisappeared(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void NetworkManager::Device::capabilitiesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void NetworkManager::Device::dhcp4ConfigChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void NetworkManager::Device::dhcp6ConfigChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void NetworkManager::Device::driverChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void NetworkManager::Device::driverVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void NetworkManager::Device::firmwareMissingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void NetworkManager::Device::firmwareVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void NetworkManager::Device::interfaceNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void NetworkManager::Device::ipV4AddressChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void NetworkManager::Device::ipV4ConfigChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void NetworkManager::Device::ipV6ConfigChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void NetworkManager::Device::ipInterfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void NetworkManager::Device::managedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void NetworkManager::Device::interfaceFlagsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void NetworkManager::Device::physicalPortIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void NetworkManager::Device::mtuChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void NetworkManager::Device::nmPluginMissingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void NetworkManager::Device::meteredChanged(MeteredStatus _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void NetworkManager::Device::connectionStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void NetworkManager::Device::stateReasonChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void NetworkManager::Device::udiChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}
QT_WARNING_POP
