/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<qmltypes.h>)
#  include <qmltypes.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_networkmanager()
{
    qmlRegisterModule("org.kde.networkmanager", 254, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    {
        Q_CONSTINIT static auto metaType = QQmlPrivate::metaTypeForNamespace(
            [](const QtPrivate::QMetaTypeInterface *) {return &NetworkManager::staticMetaObject;},
            "NetworkManager");
        QMetaType(&metaType).id();
    }
    qmlRegisterNamespaceAndRevisions(&NetworkManager::staticMetaObject, "org.kde.networkmanager", 254, nullptr, &NMForeign::staticMetaObject, nullptr);
    qmlRegisterEnum<NetworkManager::Status>("NetworkManager::Status");
    qmlRegisterEnum<NetworkManager::LogDomain>("NetworkManager::LogDomain");
    qmlRegisterEnum<NetworkManager::Connectivity>("NetworkManager::Connectivity");
    QT_WARNING_POP
    qmlRegisterModule("org.kde.networkmanager", 254, 254);
}

static const QQmlModuleRegistration orgkdenetworkmanagerRegistration("org.kde.networkmanager", qml_register_types_org_kde_networkmanager);
