/****************************************************************************
** Meta object code from reading C++ file 'wifip2ppeer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/wifip2ppeer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wifip2ppeer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14NetworkManager11WifiP2PPeerE_t {};
} // unnamed namespace

template <> constexpr inline auto NetworkManager::WifiP2PPeer::qt_create_metaobjectdata<qt_meta_tag_ZN14NetworkManager11WifiP2PPeerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NetworkManager::WifiP2PPeer",
        "flagsChanged",
        "",
        "flags",
        "hardwareAddressChanged",
        "hardwareAddress",
        "lastSeenChanged",
        "lastSeen",
        "manufacturerChanged",
        "manufacturer",
        "modelChanged",
        "model",
        "modelNumberChanged",
        "modelNumber",
        "nameChanged",
        "name",
        "serialChanged",
        "serial",
        "strengthChanged",
        "strength",
        "wfdIEsChanged",
        "wfdIEs"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'flagsChanged'
        QtMocHelpers::SignalData<void(uint)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 3 },
        }}),
        // Signal 'hardwareAddressChanged'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'lastSeenChanged'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'manufacturerChanged'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'modelChanged'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'modelNumberChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Signal 'serialChanged'
        QtMocHelpers::SignalData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Signal 'strengthChanged'
        QtMocHelpers::SignalData<void(uchar)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 19 },
        }}),
        // Signal 'wfdIEsChanged'
        QtMocHelpers::SignalData<void(const QByteArray &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'flags'
        QtMocHelpers::PropertyData<uint>(3, QMetaType::UInt, QMC::DefaultPropertyFlags, 0),
        // property 'hardwareAddress'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags, 1),
        // property 'lastSeen'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags, 2),
        // property 'manufacturer'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'model'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags, 4),
        // property 'modelNumber'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags, 6),
        // property 'serial'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags, 7),
        // property 'strength'
        QtMocHelpers::PropertyData<uchar>(19, QMetaType::UChar, QMC::DefaultPropertyFlags, 8),
        // property 'wfdIEs'
        QtMocHelpers::PropertyData<QByteArray>(21, QMetaType::QByteArray, QMC::DefaultPropertyFlags, 9),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WifiP2PPeer, qt_meta_tag_ZN14NetworkManager11WifiP2PPeerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NetworkManager::WifiP2PPeer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager11WifiP2PPeerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager11WifiP2PPeerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14NetworkManager11WifiP2PPeerE_t>.metaTypes,
    nullptr
} };

void NetworkManager::WifiP2PPeer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WifiP2PPeer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->flagsChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 1: _t->hardwareAddressChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->lastSeenChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->manufacturerChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->modelChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->modelNumberChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->nameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->serialChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->strengthChanged((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1]))); break;
        case 9: _t->wfdIEsChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WifiP2PPeer::*)(uint )>(_a, &WifiP2PPeer::flagsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WifiP2PPeer::*)(const QString & )>(_a, &WifiP2PPeer::hardwareAddressChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WifiP2PPeer::*)(int )>(_a, &WifiP2PPeer::lastSeenChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WifiP2PPeer::*)(const QString & )>(_a, &WifiP2PPeer::manufacturerChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WifiP2PPeer::*)(const QString & )>(_a, &WifiP2PPeer::modelChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WifiP2PPeer::*)(const QString & )>(_a, &WifiP2PPeer::modelNumberChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WifiP2PPeer::*)(const QString & )>(_a, &WifiP2PPeer::nameChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WifiP2PPeer::*)(const QString & )>(_a, &WifiP2PPeer::serialChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WifiP2PPeer::*)(uchar )>(_a, &WifiP2PPeer::strengthChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (WifiP2PPeer::*)(const QByteArray & )>(_a, &WifiP2PPeer::wfdIEsChanged, 9))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->flags(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->hardwareAddress(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->lastSeen(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->manufacturer(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->model(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->modelNumber(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->serial(); break;
        case 8: *reinterpret_cast<uchar*>(_v) = _t->strength(); break;
        case 9: *reinterpret_cast<QByteArray*>(_v) = _t->wfdIEs(); break;
        default: break;
        }
    }
}

const QMetaObject *NetworkManager::WifiP2PPeer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NetworkManager::WifiP2PPeer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager11WifiP2PPeerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int NetworkManager::WifiP2PPeer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void NetworkManager::WifiP2PPeer::flagsChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NetworkManager::WifiP2PPeer::hardwareAddressChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void NetworkManager::WifiP2PPeer::lastSeenChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void NetworkManager::WifiP2PPeer::manufacturerChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void NetworkManager::WifiP2PPeer::modelChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void NetworkManager::WifiP2PPeer::modelNumberChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void NetworkManager::WifiP2PPeer::nameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void NetworkManager::WifiP2PPeer::serialChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void NetworkManager::WifiP2PPeer::strengthChanged(uchar _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void NetworkManager::WifiP2PPeer::wfdIEsChanged(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
