/****************************************************************************
** Meta object code from reading C++ file 'manager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/manager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'manager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14NetworkManager8NotifierE_t {};
} // unnamed namespace

template <> constexpr inline auto NetworkManager::Notifier::qt_create_metaobjectdata<qt_meta_tag_ZN14NetworkManager8NotifierE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NetworkManager::Notifier",
        "statusChanged",
        "",
        "NetworkManager::Status",
        "status",
        "deviceAdded",
        "uni",
        "deviceRemoved",
        "wirelessEnabledChanged",
        "wwanEnabledChanged",
        "wimaxEnabledChanged",
        "wirelessHardwareEnabledChanged",
        "wwanHardwareEnabledChanged",
        "wimaxHardwareEnabledChanged",
        "networkingEnabledChanged",
        "activeConnectionAdded",
        "path",
        "activeConnectionRemoved",
        "activeConnectionsChanged",
        "serviceDisappeared",
        "serviceAppeared",
        "connectivityChanged",
        "NetworkManager::Connectivity",
        "connectivity",
        "primaryConnectionChanged",
        "activatingConnectionChanged",
        "primaryConnectionTypeChanged",
        "NetworkManager::ConnectionSettings::ConnectionType",
        "type",
        "isStartingUpChanged",
        "meteredChanged",
        "NetworkManager::Device::MeteredStatus",
        "metered",
        "globalDnsConfigurationChanged",
        "NetworkManager::DnsConfiguration",
        "configuration",
        "isConnectivityCheckAvailableChanged",
        "isConnectivityCheckEnabledChanged",
        "connectivityCheckUriChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void(NetworkManager::Status)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'deviceAdded'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'deviceRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'wirelessEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'wwanEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'wimaxEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'wirelessHardwareEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'wwanHardwareEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'wimaxHardwareEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'networkingEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'activeConnectionAdded'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Signal 'activeConnectionRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Signal 'activeConnectionsChanged'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serviceDisappeared'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serviceAppeared'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'connectivityChanged'
        QtMocHelpers::SignalData<void(NetworkManager::Connectivity)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'primaryConnectionChanged'
        QtMocHelpers::SignalData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'activatingConnectionChanged'
        QtMocHelpers::SignalData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'primaryConnectionTypeChanged'
        QtMocHelpers::SignalData<void(NetworkManager::ConnectionSettings::ConnectionType)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Signal 'isStartingUpChanged'
        QtMocHelpers::SignalData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'meteredChanged'
        QtMocHelpers::SignalData<void(NetworkManager::Device::MeteredStatus)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Signal 'globalDnsConfigurationChanged'
        QtMocHelpers::SignalData<void(const NetworkManager::DnsConfiguration &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 35 },
        }}),
        // Signal 'isConnectivityCheckAvailableChanged'
        QtMocHelpers::SignalData<void(bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'isConnectivityCheckEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'connectivityCheckUriChanged'
        QtMocHelpers::SignalData<void(QString)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Notifier, qt_meta_tag_ZN14NetworkManager8NotifierE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NetworkManager::Notifier::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager8NotifierE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager8NotifierE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14NetworkManager8NotifierE_t>.metaTypes,
    nullptr
} };

void NetworkManager::Notifier::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Notifier *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->statusChanged((*reinterpret_cast<std::add_pointer_t<NetworkManager::Status>>(_a[1]))); break;
        case 1: _t->deviceAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->deviceRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->wirelessEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->wwanEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->wimaxEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->wirelessHardwareEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->wwanHardwareEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->wimaxHardwareEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->networkingEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->activeConnectionAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->activeConnectionRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->activeConnectionsChanged(); break;
        case 13: _t->serviceDisappeared(); break;
        case 14: _t->serviceAppeared(); break;
        case 15: _t->connectivityChanged((*reinterpret_cast<std::add_pointer_t<NetworkManager::Connectivity>>(_a[1]))); break;
        case 16: _t->primaryConnectionChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->activatingConnectionChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->primaryConnectionTypeChanged((*reinterpret_cast<std::add_pointer_t<NetworkManager::ConnectionSettings::ConnectionType>>(_a[1]))); break;
        case 19: _t->isStartingUpChanged(); break;
        case 20: _t->meteredChanged((*reinterpret_cast<std::add_pointer_t<NetworkManager::Device::MeteredStatus>>(_a[1]))); break;
        case 21: _t->globalDnsConfigurationChanged((*reinterpret_cast<std::add_pointer_t<NetworkManager::DnsConfiguration>>(_a[1]))); break;
        case 22: _t->isConnectivityCheckAvailableChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->isConnectivityCheckEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->connectivityCheckUriChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(NetworkManager::Status )>(_a, &Notifier::statusChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(const QString & )>(_a, &Notifier::deviceAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(const QString & )>(_a, &Notifier::deviceRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(bool )>(_a, &Notifier::wirelessEnabledChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(bool )>(_a, &Notifier::wwanEnabledChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(bool )>(_a, &Notifier::wimaxEnabledChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(bool )>(_a, &Notifier::wirelessHardwareEnabledChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(bool )>(_a, &Notifier::wwanHardwareEnabledChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(bool )>(_a, &Notifier::wimaxHardwareEnabledChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(bool )>(_a, &Notifier::networkingEnabledChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(const QString & )>(_a, &Notifier::activeConnectionAdded, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(const QString & )>(_a, &Notifier::activeConnectionRemoved, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)()>(_a, &Notifier::activeConnectionsChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)()>(_a, &Notifier::serviceDisappeared, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)()>(_a, &Notifier::serviceAppeared, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(NetworkManager::Connectivity )>(_a, &Notifier::connectivityChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(const QString & )>(_a, &Notifier::primaryConnectionChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(const QString & )>(_a, &Notifier::activatingConnectionChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(NetworkManager::ConnectionSettings::ConnectionType )>(_a, &Notifier::primaryConnectionTypeChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)()>(_a, &Notifier::isStartingUpChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(NetworkManager::Device::MeteredStatus )>(_a, &Notifier::meteredChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(const NetworkManager::DnsConfiguration & )>(_a, &Notifier::globalDnsConfigurationChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(bool )>(_a, &Notifier::isConnectivityCheckAvailableChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(bool )>(_a, &Notifier::isConnectivityCheckEnabledChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notifier::*)(QString )>(_a, &Notifier::connectivityCheckUriChanged, 24))
            return;
    }
}

const QMetaObject *NetworkManager::Notifier::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NetworkManager::Notifier::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager8NotifierE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int NetworkManager::Notifier::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void NetworkManager::Notifier::statusChanged(NetworkManager::Status _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NetworkManager::Notifier::deviceAdded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void NetworkManager::Notifier::deviceRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void NetworkManager::Notifier::wirelessEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void NetworkManager::Notifier::wwanEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void NetworkManager::Notifier::wimaxEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void NetworkManager::Notifier::wirelessHardwareEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void NetworkManager::Notifier::wwanHardwareEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void NetworkManager::Notifier::wimaxHardwareEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void NetworkManager::Notifier::networkingEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void NetworkManager::Notifier::activeConnectionAdded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void NetworkManager::Notifier::activeConnectionRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void NetworkManager::Notifier::activeConnectionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void NetworkManager::Notifier::serviceDisappeared()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void NetworkManager::Notifier::serviceAppeared()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void NetworkManager::Notifier::connectivityChanged(NetworkManager::Connectivity _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void NetworkManager::Notifier::primaryConnectionChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void NetworkManager::Notifier::activatingConnectionChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void NetworkManager::Notifier::primaryConnectionTypeChanged(NetworkManager::ConnectionSettings::ConnectionType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void NetworkManager::Notifier::isStartingUpChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void NetworkManager::Notifier::meteredChanged(NetworkManager::Device::MeteredStatus _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void NetworkManager::Notifier::globalDnsConfigurationChanged(const NetworkManager::DnsConfiguration & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void NetworkManager::Notifier::isConnectivityCheckAvailableChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void NetworkManager::Notifier::isConnectivityCheckEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void NetworkManager::Notifier::connectivityCheckUriChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN14NetworkManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto NetworkManager::qt_create_metaobjectdata<qt_meta_tag_ZN14NetworkManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NetworkManager",
        "Status",
        "Unknown",
        "Asleep",
        "Disconnected",
        "Disconnecting",
        "Connecting",
        "ConnectedLinkLocal",
        "ConnectedSiteOnly",
        "Connected",
        "LogDomain",
        "NoChange",
        "None",
        "Hardware",
        "RFKill",
        "Ethernet",
        "WiFi",
        "Bluetooth",
        "MobileBroadBand",
        "DHCP4",
        "DHCP6",
        "PPP",
        "WiFiScan",
        "IPv4",
        "IPv6",
        "AutoIPv4",
        "DNS",
        "VPN",
        "Sharing",
        "Supplicant",
        "UserSet",
        "SysSet",
        "Suspend",
        "Core",
        "Devices",
        "OLPC",
        "Wimax",
        "Infiniband",
        "Firewall",
        "Adsl",
        "Bond",
        "Vlan",
        "Agents",
        "Settings",
        "Bridge",
        "DbusProps",
        "Team",
        "ConCheck",
        "Dcb",
        "Dispatch",
        "Connectivity",
        "UnknownConnectivity",
        "NoConnectivity",
        "Portal",
        "Limited",
        "Full"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Status'
        QtMocHelpers::EnumData<Status>(1, 1, QMC::EnumFlags{}).add({
            {    2, Status::Unknown },
            {    3, Status::Asleep },
            {    4, Status::Disconnected },
            {    5, Status::Disconnecting },
            {    6, Status::Connecting },
            {    7, Status::ConnectedLinkLocal },
            {    8, Status::ConnectedSiteOnly },
            {    9, Status::Connected },
        }),
        // enum 'LogDomain'
        QtMocHelpers::EnumData<LogDomain>(10, 10, QMC::EnumFlags{}).add({
            {   11, LogDomain::NoChange },
            {   12, LogDomain::None },
            {   13, LogDomain::Hardware },
            {   14, LogDomain::RFKill },
            {   15, LogDomain::Ethernet },
            {   16, LogDomain::WiFi },
            {   17, LogDomain::Bluetooth },
            {   18, LogDomain::MobileBroadBand },
            {   19, LogDomain::DHCP4 },
            {   20, LogDomain::DHCP6 },
            {   21, LogDomain::PPP },
            {   22, LogDomain::WiFiScan },
            {   23, LogDomain::IPv4 },
            {   24, LogDomain::IPv6 },
            {   25, LogDomain::AutoIPv4 },
            {   26, LogDomain::DNS },
            {   27, LogDomain::VPN },
            {   28, LogDomain::Sharing },
            {   29, LogDomain::Supplicant },
            {   30, LogDomain::UserSet },
            {   31, LogDomain::SysSet },
            {   32, LogDomain::Suspend },
            {   33, LogDomain::Core },
            {   34, LogDomain::Devices },
            {   35, LogDomain::OLPC },
            {   36, LogDomain::Wimax },
            {   37, LogDomain::Infiniband },
            {   38, LogDomain::Firewall },
            {   39, LogDomain::Adsl },
            {   40, LogDomain::Bond },
            {   41, LogDomain::Vlan },
            {   42, LogDomain::Agents },
            {   43, LogDomain::Settings },
            {   44, LogDomain::Bridge },
            {   45, LogDomain::DbusProps },
            {   46, LogDomain::Team },
            {   47, LogDomain::ConCheck },
            {   48, LogDomain::Dcb },
            {   49, LogDomain::Dispatch },
        }),
        // enum 'Connectivity'
        QtMocHelpers::EnumData<Connectivity>(50, 50, QMC::EnumFlags{}).add({
            {   51, Connectivity::UnknownConnectivity },
            {   52, Connectivity::NoConnectivity },
            {   53, Connectivity::Portal },
            {   54, Connectivity::Limited },
            {   55, Connectivity::Full },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN14NetworkManagerE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN14NetworkManagerE =
    NetworkManager::qt_create_metaobjectdata<qt_meta_tag_ZN14NetworkManagerE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN14NetworkManagerE =
    qt_staticMetaObjectContent_ZN14NetworkManagerE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN14NetworkManagerE =
    qt_staticMetaObjectContent_ZN14NetworkManagerE.relocatingData;

Q_CONSTINIT const QMetaObject NetworkManager::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN14NetworkManagerE.stringdata,
    qt_staticMetaObjectStaticContent_ZN14NetworkManagerE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN14NetworkManagerE.metaTypes,
    nullptr
} };

QT_WARNING_POP
