/****************************************************************************
** Meta object code from reading C++ file 'secretagentadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/dbus/secretagentadaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'secretagentadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18SecretAgentAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto SecretAgentAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN18SecretAgentAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SecretAgentAdaptor",
        "D-Bus Interface",
        "org.freedesktop.NetworkManager.SecretAgent",
        "D-Bus Introspection",
        "  <interface name=\"org.freedesktop.NetworkManager.SecretAgent\">\n"
        "    <method name=\"GetSecrets\">\n      <annotation value=\"impl_s"
        "ecret_agent_get_secrets\" name=\"org.freedesktop.DBus.GLib.CSymbol"
        "\"/>\n      <annotation value=\"\" name=\"org.freedesktop.DBus.GLi"
        "b.Async\"/>\n      <annotation value=\"NMVariantMapMap\" name=\"or"
        "g.qtproject.QtDBus.QtTypeName.In0\"/>\n      <arg direction=\"in\""
        " type=\"a{sa{sv}}\" name=\"connection\"/>\n      <arg direction=\""
        "in\" type=\"o\" name=\"connection_path\"/>\n      <arg direction=\""
        "in\" type=\"s\" name=\"setting_name\"/>\n      <arg direction=\"in"
        "\" type=\"as\" name=\"hints\"/>\n      <arg direction=\"in\" type="
        "\"u\" name=\"flags\"/>\n      <annotation value=\"NMVariantMapMap\""
        " name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n      <arg direc"
        "tion=\"out\" type=\"a{sa{sv}}\" name=\"secrets\"/>\n    </method>\n"
        "    <method name=\"CancelGetSecrets\">\n      <annotation value=\""
        "impl_secret_agent_cancel_get_secrets\" name=\"org.freedesktop.DBus"
        ".GLib.CSymbol\"/>\n      <annotation value=\"\" name=\"org.freedes"
        "ktop.DBus.GLib.Async\"/>\n      <arg direction=\"in\" type=\"o\" n"
        "ame=\"connection_path\"/>\n      <arg direction=\"in\" type=\"s\" "
        "name=\"setting_name\"/>\n    </method>\n    <method name=\"SaveSec"
        "rets\">\n      <annotation value=\"impl_secret_agent_save_secrets\""
        " name=\"org.freedesktop.DBus.GLib.CSymbol\"/>\n      <annotation v"
        "alue=\"\" name=\"org.freedesktop.DBus.GLib.Async\"/>\n      <annot"
        "ation value=\"NMVariantMapMap\" name=\"org.qtproject.QtDBus.QtType"
        "Name.In0\"/>\n      <arg direction=\"in\" type=\"a{sa{sv}}\" name="
        "\"connection\"/>\n      <arg direction=\"in\" type=\"o\" name=\"co"
        "nnection_path\"/>\n    </method>\n    <method name=\"DeleteSecrets"
        "\">\n      <annotation value=\"impl_secret_agent_delete_secrets\" "
        "name=\"org.freedesktop.DBus.GLib.CSymbol\"/>\n      <annotation va"
        "lue=\"\" name=\"org.freedesktop.DBus.GLib.Async\"/>\n      <annota"
        "tion value=\"NMVariantMapMap\" name=\"org.qtproject.QtDBus.QtTypeN"
        "ame.In0\"/>\n      <arg direction=\"in\" type=\"a{sa{sv}}\" name=\""
        "connection\"/>\n      <arg direction=\"in\" type=\"o\" name=\"conn"
        "ection_path\"/>\n    </method>\n  </interface>\n",
        "CancelGetSecrets",
        "",
        "QDBusObjectPath",
        "connection_path",
        "setting_name",
        "DeleteSecrets",
        "NMVariantMapMap",
        "connection",
        "GetSecrets",
        "hints",
        "flags",
        "SaveSecrets"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'CancelGetSecrets'
        QtMocHelpers::SlotData<void(const QDBusObjectPath &, const QString &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { QMetaType::QString, 9 },
        }}),
        // Slot 'DeleteSecrets'
        QtMocHelpers::SlotData<void(NMVariantMapMap, const QDBusObjectPath &)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'GetSecrets'
        QtMocHelpers::SlotData<NMVariantMapMap(NMVariantMapMap, const QDBusObjectPath &, const QString &, const QStringList &, uint)>(13, 6, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 7, 8 }, { QMetaType::QString, 9 }, { QMetaType::QStringList, 14 },
            { QMetaType::UInt, 15 },
        }}),
        // Slot 'SaveSecrets'
        QtMocHelpers::SlotData<void(NMVariantMapMap, const QDBusObjectPath &)>(16, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 7, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<SecretAgentAdaptor, qt_meta_tag_ZN18SecretAgentAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject SecretAgentAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18SecretAgentAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18SecretAgentAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18SecretAgentAdaptorE_t>.metaTypes,
    nullptr
} };

void SecretAgentAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SecretAgentAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->CancelGetSecrets((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->DeleteSecrets((*reinterpret_cast<std::add_pointer_t<NMVariantMapMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 2: { NMVariantMapMap _r = _t->GetSecrets((*reinterpret_cast<std::add_pointer_t<NMVariantMapMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[5])));
            if (_a[0]) *reinterpret_cast<NMVariantMapMap*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->SaveSecrets((*reinterpret_cast<std::add_pointer_t<NMVariantMapMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NMVariantMapMap >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NMVariantMapMap >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NMVariantMapMap >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
}

const QMetaObject *SecretAgentAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SecretAgentAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18SecretAgentAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int SecretAgentAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}
QT_WARNING_POP
