/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.NetworkManager.WifiP2PPeer.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "wifip2ppeerinterface.h"

/*
 * Implementation of interface class OrgFreedesktopNetworkManagerWifiP2PPeerInterface
 */

OrgFreedesktopNetworkManagerWifiP2PPeerInterface::OrgFreedesktopNetworkManagerWifiP2PPeerInterface(const QString &service,
                                                                                                   const QString &path,
                                                                                                   const QDBusConnection &connection,
                                                                                                   QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
}

OrgFreedesktopNetworkManagerWifiP2PPeerInterface::~OrgFreedesktopNetworkManagerWifiP2PPeerInterface()
{
}

#include "moc_wifip2ppeerinterface.cpp"
