/****************************************************************************
** Meta object code from reading C++ file 'accesspoint.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/accesspoint.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'accesspoint.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14NetworkManager11AccessPointE_t {};
} // unnamed namespace

template <> constexpr inline auto NetworkManager::AccessPoint::qt_create_metaobjectdata<qt_meta_tag_ZN14NetworkManager11AccessPointE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NetworkManager::AccessPoint",
        "signalStrengthChanged",
        "",
        "strength",
        "bitRateChanged",
        "bitrate",
        "capabilitiesChanged",
        "AccessPoint::Capabilities",
        "caps",
        "wpaFlagsChanged",
        "AccessPoint::WpaFlags",
        "flags",
        "rsnFlagsChanged",
        "ssidChanged",
        "ssid",
        "frequencyChanged",
        "frequency",
        "lastSeenChanged",
        "lastSeen",
        "bandwidthChanged",
        "bandwidth",
        "Capabilities",
        "Capability",
        "None",
        "Privacy",
        "WpaFlags",
        "WpaFlag",
        "PairWep40",
        "PairWep104",
        "PairTkip",
        "PairCcmp",
        "GroupWep40",
        "GroupWep104",
        "GroupTkip",
        "GroupCcmp",
        "KeyMgmtPsk",
        "KeyMgmt8021x",
        "KeyMgmtSAE",
        "KeyMgmtOWE",
        "KeyMgmtOWETM",
        "KeyMgmtEapSuiteB192"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalStrengthChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'bitRateChanged'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'capabilitiesChanged'
        QtMocHelpers::SignalData<void(AccessPoint::Capabilities)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'wpaFlagsChanged'
        QtMocHelpers::SignalData<void(AccessPoint::WpaFlags)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'rsnFlagsChanged'
        QtMocHelpers::SignalData<void(AccessPoint::WpaFlags)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'ssidChanged'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Signal 'frequencyChanged'
        QtMocHelpers::SignalData<void(uint)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 16 },
        }}),
        // Signal 'lastSeenChanged'
        QtMocHelpers::SignalData<void(int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 },
        }}),
        // Signal 'bandwidthChanged'
        QtMocHelpers::SignalData<void(uint)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'Capabilities'
        QtMocHelpers::EnumData<Capabilities>(21, 22, QMC::EnumIsFlag).add({
            {   23, Capability::None },
            {   24, Capability::Privacy },
        }),
        // flag 'WpaFlags'
        QtMocHelpers::EnumData<WpaFlags>(25, 26, QMC::EnumIsFlag).add({
            {   27, WpaFlag::PairWep40 },
            {   28, WpaFlag::PairWep104 },
            {   29, WpaFlag::PairTkip },
            {   30, WpaFlag::PairCcmp },
            {   31, WpaFlag::GroupWep40 },
            {   32, WpaFlag::GroupWep104 },
            {   33, WpaFlag::GroupTkip },
            {   34, WpaFlag::GroupCcmp },
            {   35, WpaFlag::KeyMgmtPsk },
            {   36, WpaFlag::KeyMgmt8021x },
            {   37, WpaFlag::KeyMgmtSAE },
            {   38, WpaFlag::KeyMgmtOWE },
            {   39, WpaFlag::KeyMgmtOWETM },
            {   40, WpaFlag::KeyMgmtEapSuiteB192 },
        }),
    };
    return QtMocHelpers::metaObjectData<AccessPoint, qt_meta_tag_ZN14NetworkManager11AccessPointE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NetworkManager::AccessPoint::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager11AccessPointE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager11AccessPointE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14NetworkManager11AccessPointE_t>.metaTypes,
    nullptr
} };

void NetworkManager::AccessPoint::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccessPoint *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalStrengthChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->bitRateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->capabilitiesChanged((*reinterpret_cast<std::add_pointer_t<AccessPoint::Capabilities>>(_a[1]))); break;
        case 3: _t->wpaFlagsChanged((*reinterpret_cast<std::add_pointer_t<AccessPoint::WpaFlags>>(_a[1]))); break;
        case 4: _t->rsnFlagsChanged((*reinterpret_cast<std::add_pointer_t<AccessPoint::WpaFlags>>(_a[1]))); break;
        case 5: _t->ssidChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->frequencyChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 7: _t->lastSeenChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->bandwidthChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AccessPoint::*)(int )>(_a, &AccessPoint::signalStrengthChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccessPoint::*)(int )>(_a, &AccessPoint::bitRateChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccessPoint::*)(AccessPoint::Capabilities )>(_a, &AccessPoint::capabilitiesChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccessPoint::*)(AccessPoint::WpaFlags )>(_a, &AccessPoint::wpaFlagsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccessPoint::*)(AccessPoint::WpaFlags )>(_a, &AccessPoint::rsnFlagsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccessPoint::*)(const QString & )>(_a, &AccessPoint::ssidChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccessPoint::*)(uint )>(_a, &AccessPoint::frequencyChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccessPoint::*)(int )>(_a, &AccessPoint::lastSeenChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccessPoint::*)(uint )>(_a, &AccessPoint::bandwidthChanged, 8))
            return;
    }
}

const QMetaObject *NetworkManager::AccessPoint::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NetworkManager::AccessPoint::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager11AccessPointE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int NetworkManager::AccessPoint::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void NetworkManager::AccessPoint::signalStrengthChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NetworkManager::AccessPoint::bitRateChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void NetworkManager::AccessPoint::capabilitiesChanged(AccessPoint::Capabilities _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void NetworkManager::AccessPoint::wpaFlagsChanged(AccessPoint::WpaFlags _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void NetworkManager::AccessPoint::rsnFlagsChanged(AccessPoint::WpaFlags _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void NetworkManager::AccessPoint::ssidChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void NetworkManager::AccessPoint::frequencyChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void NetworkManager::AccessPoint::lastSeenChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void NetworkManager::AccessPoint::bandwidthChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
