/****************************************************************************
** Meta object code from reading C++ file 'networkmanagerinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/dbus/networkmanagerinterface.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'networkmanagerinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN37OrgFreedesktopNetworkManagerInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgFreedesktopNetworkManagerInterface::qt_create_metaobjectdata<qt_meta_tag_ZN37OrgFreedesktopNetworkManagerInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgFreedesktopNetworkManagerInterface",
        "CheckPermissions",
        "",
        "DeviceAdded",
        "QDBusObjectPath",
        "device_path",
        "DeviceRemoved",
        "PropertiesChanged",
        "QVariantMap",
        "properties",
        "StateChanged",
        "state",
        "ActivateConnection",
        "QDBusPendingReply<QDBusObjectPath>",
        "connection",
        "device",
        "specific_object",
        "AddAndActivateConnection",
        "QDBusPendingReply<QDBusObjectPath,QDBusObjectPath>",
        "NMVariantMapMap",
        "AddAndActivateConnection2",
        "QDBusPendingReply<QDBusObjectPath,QDBusObjectPath,QVariantMap>",
        "options",
        "CheckConnectivity",
        "QDBusPendingReply<uint>",
        "CheckpointCreate",
        "QList<QDBusObjectPath>",
        "devices",
        "rollback_timeout",
        "flags",
        "CheckpointDestroy",
        "QDBusPendingReply<>",
        "checkpoint",
        "DeactivateConnection",
        "active_connection",
        "Enable",
        "enable",
        "GetAllDevices",
        "QDBusPendingReply<QList<QDBusObjectPath>>",
        "GetDeviceByIpIface",
        "iface",
        "GetDevices",
        "GetLogging",
        "QDBusPendingReply<QString,QString>",
        "GetPermissions",
        "QDBusPendingReply<NMStringMap>",
        "Reload",
        "SetLogging",
        "level",
        "domains",
        "Sleep",
        "sleep",
        "ActivatingConnection",
        "ActiveConnections",
        "AllDevices",
        "Capabilities",
        "UIntList",
        "Connectivity",
        "ConnectivityCheckAvailable",
        "ConnectivityCheckEnabled",
        "ConnectivityCheckUri",
        "Devices",
        "GlobalDnsConfiguration",
        "Metered",
        "NetworkingEnabled",
        "PrimaryConnection",
        "PrimaryConnectionType",
        "Startup",
        "State",
        "Version",
        "WimaxEnabled",
        "WimaxHardwareEnabled",
        "WirelessEnabled",
        "WirelessHardwareEnabled",
        "WwanEnabled",
        "WwanHardwareEnabled"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'CheckPermissions'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'DeviceAdded'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'DeviceRemoved'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'PropertiesChanged'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'StateChanged'
        QtMocHelpers::SignalData<void(uint)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 11 },
        }}),
        // Slot 'ActivateConnection'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(const QDBusObjectPath &, const QDBusObjectPath &, const QDBusObjectPath &)>(12, 2, QMC::AccessPublic, 0x80000000 | 13, {{
            { 0x80000000 | 4, 14 }, { 0x80000000 | 4, 15 }, { 0x80000000 | 4, 16 },
        }}),
        // Slot 'AddAndActivateConnection'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath,QDBusObjectPath>(NMVariantMapMap, const QDBusObjectPath &, const QDBusObjectPath &)>(17, 2, QMC::AccessPublic, 0x80000000 | 18, {{
            { 0x80000000 | 19, 14 }, { 0x80000000 | 4, 15 }, { 0x80000000 | 4, 16 },
        }}),
        // Slot 'AddAndActivateConnection2'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath,QDBusObjectPath,QVariantMap>(NMVariantMapMap, const QDBusObjectPath &, const QDBusObjectPath &, const QVariantMap &)>(20, 2, QMC::AccessPublic, 0x80000000 | 21, {{
            { 0x80000000 | 19, 14 }, { 0x80000000 | 4, 15 }, { 0x80000000 | 4, 16 }, { 0x80000000 | 8, 22 },
        }}),
        // Slot 'CheckConnectivity'
        QtMocHelpers::SlotData<QDBusPendingReply<uint>()>(23, 2, QMC::AccessPublic, 0x80000000 | 24),
        // Slot 'CheckpointCreate'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(const QList<QDBusObjectPath> &, uint, uint)>(25, 2, QMC::AccessPublic, 0x80000000 | 13, {{
            { 0x80000000 | 26, 27 }, { QMetaType::UInt, 28 }, { QMetaType::UInt, 29 },
        }}),
        // Slot 'CheckpointDestroy'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &)>(30, 2, QMC::AccessPublic, 0x80000000 | 31, {{
            { 0x80000000 | 4, 32 },
        }}),
        // Slot 'DeactivateConnection'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &)>(33, 2, QMC::AccessPublic, 0x80000000 | 31, {{
            { 0x80000000 | 4, 34 },
        }}),
        // Slot 'Enable'
        QtMocHelpers::SlotData<QDBusPendingReply<>(bool)>(35, 2, QMC::AccessPublic, 0x80000000 | 31, {{
            { QMetaType::Bool, 36 },
        }}),
        // Slot 'GetAllDevices'
        QtMocHelpers::SlotData<QDBusPendingReply<QList<QDBusObjectPath>>()>(37, 2, QMC::AccessPublic, 0x80000000 | 38),
        // Slot 'GetDeviceByIpIface'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(const QString &)>(39, 2, QMC::AccessPublic, 0x80000000 | 13, {{
            { QMetaType::QString, 40 },
        }}),
        // Slot 'GetDevices'
        QtMocHelpers::SlotData<QDBusPendingReply<QList<QDBusObjectPath>>()>(41, 2, QMC::AccessPublic, 0x80000000 | 38),
        // Slot 'GetLogging'
        QtMocHelpers::SlotData<QDBusPendingReply<QString,QString>()>(42, 2, QMC::AccessPublic, 0x80000000 | 43),
        // Slot 'GetPermissions'
        QtMocHelpers::SlotData<QDBusPendingReply<NMStringMap>()>(44, 2, QMC::AccessPublic, 0x80000000 | 45),
        // Slot 'Reload'
        QtMocHelpers::SlotData<QDBusPendingReply<>(uint)>(46, 2, QMC::AccessPublic, 0x80000000 | 31, {{
            { QMetaType::UInt, 29 },
        }}),
        // Slot 'SetLogging'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, const QString &)>(47, 2, QMC::AccessPublic, 0x80000000 | 31, {{
            { QMetaType::QString, 48 }, { QMetaType::QString, 49 },
        }}),
        // Slot 'Sleep'
        QtMocHelpers::SlotData<QDBusPendingReply<>(bool)>(50, 2, QMC::AccessPublic, 0x80000000 | 31, {{
            { QMetaType::Bool, 51 },
        }}),
        // Slot 'state'
        QtMocHelpers::SlotData<QDBusPendingReply<uint>()>(11, 2, QMC::AccessPublic, 0x80000000 | 24),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'ActivatingConnection'
        QtMocHelpers::PropertyData<QDBusObjectPath>(52, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'ActiveConnections'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(53, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'AllDevices'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(54, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Capabilities'
        QtMocHelpers::PropertyData<UIntList>(55, 0x80000000 | 56, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Connectivity'
        QtMocHelpers::PropertyData<uint>(57, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'ConnectivityCheckAvailable'
        QtMocHelpers::PropertyData<bool>(58, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'ConnectivityCheckEnabled'
        QtMocHelpers::PropertyData<bool>(59, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'ConnectivityCheckUri'
        QtMocHelpers::PropertyData<QString>(60, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Devices'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(61, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'GlobalDnsConfiguration'
        QtMocHelpers::PropertyData<QVariantMap>(62, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'Metered'
        QtMocHelpers::PropertyData<uint>(63, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'NetworkingEnabled'
        QtMocHelpers::PropertyData<bool>(64, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'PrimaryConnection'
        QtMocHelpers::PropertyData<QDBusObjectPath>(65, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'PrimaryConnectionType'
        QtMocHelpers::PropertyData<QString>(66, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Startup'
        QtMocHelpers::PropertyData<bool>(67, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'State'
        QtMocHelpers::PropertyData<uint>(68, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'Version'
        QtMocHelpers::PropertyData<QString>(69, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'WimaxEnabled'
        QtMocHelpers::PropertyData<bool>(70, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'WimaxHardwareEnabled'
        QtMocHelpers::PropertyData<bool>(71, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'WirelessEnabled'
        QtMocHelpers::PropertyData<bool>(72, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'WirelessHardwareEnabled'
        QtMocHelpers::PropertyData<bool>(73, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'WwanEnabled'
        QtMocHelpers::PropertyData<bool>(74, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'WwanHardwareEnabled'
        QtMocHelpers::PropertyData<bool>(75, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopNetworkManagerInterface, qt_meta_tag_ZN37OrgFreedesktopNetworkManagerInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgFreedesktopNetworkManagerInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37OrgFreedesktopNetworkManagerInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37OrgFreedesktopNetworkManagerInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN37OrgFreedesktopNetworkManagerInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgFreedesktopNetworkManagerInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopNetworkManagerInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->CheckPermissions(); break;
        case 1: _t->DeviceAdded((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 2: _t->DeviceRemoved((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 3: _t->PropertiesChanged((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 4: _t->StateChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 5: { QDBusPendingReply<QDBusObjectPath> _r = _t->ActivateConnection((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<QDBusObjectPath,QDBusObjectPath> _r = _t->AddAndActivateConnection((*reinterpret_cast<std::add_pointer_t<NMVariantMapMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath,QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<QDBusObjectPath,QDBusObjectPath,QVariantMap> _r = _t->AddAndActivateConnection2((*reinterpret_cast<std::add_pointer_t<NMVariantMapMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath,QDBusObjectPath,QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<uint> _r = _t->CheckConnectivity();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<uint>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<QDBusObjectPath> _r = _t->CheckpointCreate((*reinterpret_cast<std::add_pointer_t<QList<QDBusObjectPath>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<> _r = _t->CheckpointDestroy((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<> _r = _t->DeactivateConnection((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<> _r = _t->Enable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QDBusPendingReply<QList<QDBusObjectPath>> _r = _t->GetAllDevices();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QList<QDBusObjectPath>>*>(_a[0]) = std::move(_r); }  break;
        case 14: { QDBusPendingReply<QDBusObjectPath> _r = _t->GetDeviceByIpIface((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 15: { QDBusPendingReply<QList<QDBusObjectPath>> _r = _t->GetDevices();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QList<QDBusObjectPath>>*>(_a[0]) = std::move(_r); }  break;
        case 16: { QDBusPendingReply<QString,QString> _r = _t->GetLogging();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString,QString>*>(_a[0]) = std::move(_r); }  break;
        case 17: { QDBusPendingReply<NMStringMap> _r = _t->GetPermissions();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<NMStringMap>*>(_a[0]) = std::move(_r); }  break;
        case 18: { QDBusPendingReply<> _r = _t->Reload((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 19: { QDBusPendingReply<> _r = _t->SetLogging((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 20: { QDBusPendingReply<> _r = _t->Sleep((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 21: { QDBusPendingReply<uint> _r = _t->state();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<uint>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NMVariantMapMap >(); break;
            case 2:
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NMVariantMapMap >(); break;
            case 2:
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QDBusObjectPath> >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerInterface::*)()>(_a, &OrgFreedesktopNetworkManagerInterface::CheckPermissions, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerInterface::*)(const QDBusObjectPath & )>(_a, &OrgFreedesktopNetworkManagerInterface::DeviceAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerInterface::*)(const QDBusObjectPath & )>(_a, &OrgFreedesktopNetworkManagerInterface::DeviceRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerInterface::*)(const QVariantMap & )>(_a, &OrgFreedesktopNetworkManagerInterface::PropertiesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerInterface::*)(uint )>(_a, &OrgFreedesktopNetworkManagerInterface::StateChanged, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 12:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
        case 8:
        case 2:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QDBusObjectPath> >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< UIntList >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->activatingConnection(); break;
        case 1: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->activeConnections(); break;
        case 2: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->allDevices(); break;
        case 3: *reinterpret_cast<UIntList*>(_v) = _t->capabilities(); break;
        case 4: *reinterpret_cast<uint*>(_v) = _t->connectivity(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->connectivityCheckAvailable(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->connectivityCheckEnabled(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->connectivityCheckUri(); break;
        case 8: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->devices(); break;
        case 9: *reinterpret_cast<QVariantMap*>(_v) = _t->globalDnsConfiguration(); break;
        case 10: *reinterpret_cast<uint*>(_v) = _t->metered(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->networkingEnabled(); break;
        case 12: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->primaryConnection(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->primaryConnectionType(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->startup(); break;
        case 15: *reinterpret_cast<uint*>(_v) = _t->state(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->version(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->wimaxEnabled(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->wimaxHardwareEnabled(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->wirelessEnabled(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->wirelessHardwareEnabled(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->wwanEnabled(); break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->wwanHardwareEnabled(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 9: _t->setGlobalDnsConfiguration(*reinterpret_cast<QVariantMap*>(_v)); break;
        case 17: _t->setWimaxEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 19: _t->setWirelessEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 21: _t->setWwanEnabled(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OrgFreedesktopNetworkManagerInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgFreedesktopNetworkManagerInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37OrgFreedesktopNetworkManagerInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgFreedesktopNetworkManagerInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void OrgFreedesktopNetworkManagerInterface::CheckPermissions()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OrgFreedesktopNetworkManagerInterface::DeviceAdded(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OrgFreedesktopNetworkManagerInterface::DeviceRemoved(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OrgFreedesktopNetworkManagerInterface::PropertiesChanged(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OrgFreedesktopNetworkManagerInterface::StateChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
