/****************************************************************************
** Meta object code from reading C++ file 'wifip2pdevice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/wifip2pdevice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wifip2pdevice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14NetworkManager13WifiP2PDeviceE_t {};
} // unnamed namespace

template <> constexpr inline auto NetworkManager::WifiP2PDevice::qt_create_metaobjectdata<qt_meta_tag_ZN14NetworkManager13WifiP2PDeviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NetworkManager::WifiP2PDevice",
        "hardwareAddressChanged",
        "",
        "hardwareAddress",
        "peersChanged",
        "peers",
        "peerAppeared",
        "peer",
        "peerDisappeared",
        "startFind",
        "QDBusPendingReply<>",
        "QVariantMap",
        "options",
        "stopFind",
        "findPeer",
        "WifiP2PPeer::Ptr",
        "uni"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'hardwareAddressChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'peersChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 5 },
        }}),
        // Signal 'peerAppeared'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'peerDisappeared'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'startFind'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QVariantMap &)>(9, 2, QMC::AccessPublic, 0x80000000 | 10, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'startFind'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(9, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 10),
        // Slot 'stopFind'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(13, 2, QMC::AccessPublic, 0x80000000 | 10),
        // Method 'findPeer'
        QtMocHelpers::MethodData<WifiP2PPeer::Ptr(const QString &)>(14, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QString, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'hardwareAddress'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'peers'
        QtMocHelpers::PropertyData<QStringList>(5, QMetaType::QStringList, QMC::DefaultPropertyFlags, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WifiP2PDevice, qt_meta_tag_ZN14NetworkManager13WifiP2PDeviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NetworkManager::WifiP2PDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<Device::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager13WifiP2PDeviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager13WifiP2PDeviceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14NetworkManager13WifiP2PDeviceE_t>.metaTypes,
    nullptr
} };

void NetworkManager::WifiP2PDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WifiP2PDevice *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->hardwareAddressChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->peersChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 2: _t->peerAppeared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->peerDisappeared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: { QDBusPendingReply<> _r = _t->startFind((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->startFind();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->stopFind();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { WifiP2PPeer::Ptr _r = _t->findPeer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<WifiP2PPeer::Ptr*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WifiP2PDevice::*)(const QString & )>(_a, &WifiP2PDevice::hardwareAddressChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WifiP2PDevice::*)(const QStringList & )>(_a, &WifiP2PDevice::peersChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WifiP2PDevice::*)(const QString & )>(_a, &WifiP2PDevice::peerAppeared, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WifiP2PDevice::*)(const QString & )>(_a, &WifiP2PDevice::peerDisappeared, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->hardwareAddress(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->peers(); break;
        default: break;
        }
    }
}

const QMetaObject *NetworkManager::WifiP2PDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NetworkManager::WifiP2PDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager13WifiP2PDeviceE_t>.strings))
        return static_cast<void*>(this);
    return Device::qt_metacast(_clname);
}

int NetworkManager::WifiP2PDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Device::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void NetworkManager::WifiP2PDevice::hardwareAddressChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NetworkManager::WifiP2PDevice::peersChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void NetworkManager::WifiP2PDevice::peerAppeared(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void NetworkManager::WifiP2PDevice::peerDisappeared(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
