/****************************************************************************
** Meta object code from reading C++ file 'vpnconnection.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/vpnconnection.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vpnconnection.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14NetworkManager13VpnConnectionE_t {};
} // unnamed namespace

template <> constexpr inline auto NetworkManager::VpnConnection::qt_create_metaobjectdata<qt_meta_tag_ZN14NetworkManager13VpnConnectionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NetworkManager::VpnConnection",
        "bannerChanged",
        "",
        "banner",
        "stateChanged",
        "NetworkManager::VpnConnection::State",
        "state",
        "NetworkManager::VpnConnection::StateChangeReason",
        "reason",
        "State",
        "Unknown",
        "Prepare",
        "NeedAuth",
        "Connecting",
        "GettingIpConfig",
        "Activated",
        "Failed",
        "Disconnected",
        "StateChangeReason",
        "UnknownReason",
        "NoneReason",
        "UserDisconnectedReason",
        "DeviceDisconnectedReason",
        "ServiceStoppedReason",
        "IpConfigInvalidReason",
        "ConnectTimeoutReason",
        "ServiceStartTimeoutReason",
        "ServiceStartFailedReason",
        "NoSecretsReason",
        "LoginFailedReason",
        "ConnectionRemovedReason"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'bannerChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(NetworkManager::VpnConnection::State, NetworkManager::VpnConnection::StateChangeReason)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(9, 9, QMC::EnumFlags{}).add({
            {   10, State::Unknown },
            {   11, State::Prepare },
            {   12, State::NeedAuth },
            {   13, State::Connecting },
            {   14, State::GettingIpConfig },
            {   15, State::Activated },
            {   16, State::Failed },
            {   17, State::Disconnected },
        }),
        // enum 'StateChangeReason'
        QtMocHelpers::EnumData<enum StateChangeReason>(18, 18, QMC::EnumFlags{}).add({
            {   19, StateChangeReason::UnknownReason },
            {   20, StateChangeReason::NoneReason },
            {   21, StateChangeReason::UserDisconnectedReason },
            {   22, StateChangeReason::DeviceDisconnectedReason },
            {   23, StateChangeReason::ServiceStoppedReason },
            {   24, StateChangeReason::IpConfigInvalidReason },
            {   25, StateChangeReason::ConnectTimeoutReason },
            {   26, StateChangeReason::ServiceStartTimeoutReason },
            {   27, StateChangeReason::ServiceStartFailedReason },
            {   28, StateChangeReason::NoSecretsReason },
            {   29, StateChangeReason::LoginFailedReason },
            {   30, StateChangeReason::ConnectionRemovedReason },
        }),
    };
    return QtMocHelpers::metaObjectData<VpnConnection, qt_meta_tag_ZN14NetworkManager13VpnConnectionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NetworkManager::VpnConnection::staticMetaObject = { {
    QMetaObject::SuperData::link<ActiveConnection::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager13VpnConnectionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager13VpnConnectionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14NetworkManager13VpnConnectionE_t>.metaTypes,
    nullptr
} };

void NetworkManager::VpnConnection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<VpnConnection *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->bannerChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<NetworkManager::VpnConnection::State>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<NetworkManager::VpnConnection::StateChangeReason>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (VpnConnection::*)(const QString & )>(_a, &VpnConnection::bannerChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (VpnConnection::*)(NetworkManager::VpnConnection::State , NetworkManager::VpnConnection::StateChangeReason )>(_a, &VpnConnection::stateChanged, 1))
            return;
    }
}

const QMetaObject *NetworkManager::VpnConnection::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NetworkManager::VpnConnection::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager13VpnConnectionE_t>.strings))
        return static_cast<void*>(this);
    return ActiveConnection::qt_metacast(_clname);
}

int NetworkManager::VpnConnection::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ActiveConnection::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void NetworkManager::VpnConnection::bannerChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NetworkManager::VpnConnection::stateChanged(NetworkManager::VpnConnection::State _t1, NetworkManager::VpnConnection::StateChangeReason _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
