/****************************************************************************
** Meta object code from reading C++ file 'tundevice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/tundevice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tundevice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14NetworkManager9TunDeviceE_t {};
} // unnamed namespace

template <> constexpr inline auto NetworkManager::TunDevice::qt_create_metaobjectdata<qt_meta_tag_ZN14NetworkManager9TunDeviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NetworkManager::TunDevice",
        "ownerChanged",
        "",
        "owner",
        "groupChanged",
        "group",
        "modeChanged",
        "mode",
        "multiQueueChanged",
        "multiQueue",
        "noPiChanged",
        "noPi",
        "vnetHdrChanged",
        "vnetHdr",
        "hwAddressChanged",
        "hwAddress"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ownerChanged'
        QtMocHelpers::SignalData<void(qlonglong)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 3 },
        }}),
        // Signal 'groupChanged'
        QtMocHelpers::SignalData<void(qlonglong)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 5 },
        }}),
        // Signal 'modeChanged'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'multiQueueChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'noPiChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'vnetHdrChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'hwAddressChanged'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'owner'
        QtMocHelpers::PropertyData<qlonglong>(3, QMetaType::LongLong, QMC::DefaultPropertyFlags, 0),
        // property 'group'
        QtMocHelpers::PropertyData<qlonglong>(5, QMetaType::LongLong, QMC::DefaultPropertyFlags, 1),
        // property 'mode'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'multiQueue'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'noPi'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
        // property 'vnetHdr'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags, 5),
        // property 'hwAddress'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TunDevice, qt_meta_tag_ZN14NetworkManager9TunDeviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NetworkManager::TunDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<Device::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager9TunDeviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager9TunDeviceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14NetworkManager9TunDeviceE_t>.metaTypes,
    nullptr
} };

void NetworkManager::TunDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TunDevice *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ownerChanged((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1]))); break;
        case 1: _t->groupChanged((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1]))); break;
        case 2: _t->modeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->multiQueueChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->noPiChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->vnetHdrChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->hwAddressChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TunDevice::*)(qlonglong )>(_a, &TunDevice::ownerChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TunDevice::*)(qlonglong )>(_a, &TunDevice::groupChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TunDevice::*)(const QString & )>(_a, &TunDevice::modeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TunDevice::*)(bool )>(_a, &TunDevice::multiQueueChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TunDevice::*)(bool )>(_a, &TunDevice::noPiChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TunDevice::*)(bool )>(_a, &TunDevice::vnetHdrChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TunDevice::*)(const QString & )>(_a, &TunDevice::hwAddressChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qlonglong*>(_v) = _t->owner(); break;
        case 1: *reinterpret_cast<qlonglong*>(_v) = _t->group(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->mode(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->multiQueue(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->noPi(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->vnetHdr(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->hwAddress(); break;
        default: break;
        }
    }
}

const QMetaObject *NetworkManager::TunDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NetworkManager::TunDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager9TunDeviceE_t>.strings))
        return static_cast<void*>(this);
    return Device::qt_metacast(_clname);
}

int NetworkManager::TunDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Device::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void NetworkManager::TunDevice::ownerChanged(qlonglong _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NetworkManager::TunDevice::groupChanged(qlonglong _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void NetworkManager::TunDevice::modeChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void NetworkManager::TunDevice::multiQueueChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void NetworkManager::TunDevice::noPiChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void NetworkManager::TunDevice::vnetHdrChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void NetworkManager::TunDevice::hwAddressChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
