/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was:
 *
 * qdbusxml2cpp is Copyright (C) 2016 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "networkmanagerinterface.h"

/*
 * Implementation of interface class OrgFreedesktopNetworkManagerInterface
 */

OrgFreedesktopNetworkManagerInterface::OrgFreedesktopNetworkManagerInterface(const QString &service,
                                                                             const QString &path,
                                                                             const QDBusConnection &connection,
                                                                             QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
}

OrgFreedesktopNetworkManagerInterface::~OrgFreedesktopNetworkManagerInterface()
{
}

#include "moc_networkmanagerinterface.cpp"
