/****************************************************************************
** Meta object code from reading C++ file 'ip4configinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/dbus/ip4configinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ip4configinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN46OrgFreedesktopNetworkManagerIP4ConfigInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgFreedesktopNetworkManagerIP4ConfigInterface::qt_create_metaobjectdata<qt_meta_tag_ZN46OrgFreedesktopNetworkManagerIP4ConfigInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgFreedesktopNetworkManagerIP4ConfigInterface",
        "PropertiesChanged",
        "",
        "QVariantMap",
        "properties",
        "AddressData",
        "NMVariantMapList",
        "Addresses",
        "UIntListList",
        "DnsOptions",
        "DnsPriority",
        "Domains",
        "Gateway",
        "Nameservers",
        "UIntList",
        "RouteData",
        "Routes",
        "Searches",
        "WinsServers"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'PropertiesChanged'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'AddressData'
        QtMocHelpers::PropertyData<NMVariantMapList>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Addresses'
        QtMocHelpers::PropertyData<UIntListList>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'DnsOptions'
        QtMocHelpers::PropertyData<QStringList>(9, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'DnsPriority'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'Domains'
        QtMocHelpers::PropertyData<QStringList>(11, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'Gateway'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Nameservers'
        QtMocHelpers::PropertyData<UIntList>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'RouteData'
        QtMocHelpers::PropertyData<NMVariantMapList>(15, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Routes'
        QtMocHelpers::PropertyData<UIntListList>(16, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Searches'
        QtMocHelpers::PropertyData<QStringList>(17, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'WinsServers'
        QtMocHelpers::PropertyData<UIntList>(18, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopNetworkManagerIP4ConfigInterface, qt_meta_tag_ZN46OrgFreedesktopNetworkManagerIP4ConfigInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgFreedesktopNetworkManagerIP4ConfigInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN46OrgFreedesktopNetworkManagerIP4ConfigInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN46OrgFreedesktopNetworkManagerIP4ConfigInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN46OrgFreedesktopNetworkManagerIP4ConfigInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgFreedesktopNetworkManagerIP4ConfigInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopNetworkManagerIP4ConfigInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->PropertiesChanged((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerIP4ConfigInterface::*)(const QVariantMap & )>(_a, &OrgFreedesktopNetworkManagerIP4ConfigInterface::PropertiesChanged, 0))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< NMVariantMapList >(); break;
        case 10:
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< UIntList >(); break;
        case 8:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< UIntListList >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<NMVariantMapList*>(_v) = _t->addressData(); break;
        case 1: *reinterpret_cast<UIntListList*>(_v) = _t->addresses(); break;
        case 2: *reinterpret_cast<QStringList*>(_v) = _t->dnsOptions(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->dnsPriority(); break;
        case 4: *reinterpret_cast<QStringList*>(_v) = _t->domains(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->gateway(); break;
        case 6: *reinterpret_cast<UIntList*>(_v) = _t->nameservers(); break;
        case 7: *reinterpret_cast<NMVariantMapList*>(_v) = _t->routeData(); break;
        case 8: *reinterpret_cast<UIntListList*>(_v) = _t->routes(); break;
        case 9: *reinterpret_cast<QStringList*>(_v) = _t->searches(); break;
        case 10: *reinterpret_cast<UIntList*>(_v) = _t->winsServers(); break;
        default: break;
        }
    }
}

const QMetaObject *OrgFreedesktopNetworkManagerIP4ConfigInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgFreedesktopNetworkManagerIP4ConfigInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN46OrgFreedesktopNetworkManagerIP4ConfigInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgFreedesktopNetworkManagerIP4ConfigInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void OrgFreedesktopNetworkManagerIP4ConfigInterface::PropertiesChanged(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
