/****************************************************************************
** Meta object code from reading C++ file 'wirelessdevice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/wirelessdevice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wirelessdevice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14NetworkManager14WirelessDeviceE_t {};
} // unnamed namespace

template <> constexpr inline auto NetworkManager::WirelessDevice::qt_create_metaobjectdata<qt_meta_tag_ZN14NetworkManager14WirelessDeviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NetworkManager::WirelessDevice",
        "bitRateChanged",
        "",
        "bitrate",
        "activeAccessPointChanged",
        "modeChanged",
        "WirelessDevice::OperationMode",
        "wirelessCapabilitiesChanged",
        "Capabilities",
        "hardwareAddressChanged",
        "permanentHardwareAddressChanged",
        "wirelessPropertiesChanged",
        "accessPointAppeared",
        "uni",
        "accessPointDisappeared",
        "networkAppeared",
        "ssid",
        "networkDisappeared",
        "lastScanChanged",
        "dateTime",
        "OperationMode",
        "Unknown",
        "Adhoc",
        "Infra",
        "ApMode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'bitRateChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'activeAccessPointChanged'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'modeChanged'
        QtMocHelpers::SignalData<void(WirelessDevice::OperationMode)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Signal 'wirelessCapabilitiesChanged'
        QtMocHelpers::SignalData<void(Capabilities)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Signal 'hardwareAddressChanged'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'permanentHardwareAddressChanged'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'wirelessPropertiesChanged'
        QtMocHelpers::SignalData<void(uint)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 2 },
        }}),
        // Signal 'accessPointAppeared'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'accessPointDisappeared'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'networkAppeared'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Signal 'networkDisappeared'
        QtMocHelpers::SignalData<void(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Signal 'lastScanChanged'
        QtMocHelpers::SignalData<void(const QDateTime &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDateTime, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'OperationMode'
        QtMocHelpers::EnumData<enum OperationMode>(20, 20, QMC::EnumFlags{}).add({
            {   21, OperationMode::Unknown },
            {   22, OperationMode::Adhoc },
            {   23, OperationMode::Infra },
            {   24, OperationMode::ApMode },
        }),
    };
    return QtMocHelpers::metaObjectData<WirelessDevice, qt_meta_tag_ZN14NetworkManager14WirelessDeviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NetworkManager::WirelessDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<Device::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager14WirelessDeviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager14WirelessDeviceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14NetworkManager14WirelessDeviceE_t>.metaTypes,
    nullptr
} };

void NetworkManager::WirelessDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WirelessDevice *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->bitRateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->activeAccessPointChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->modeChanged((*reinterpret_cast<std::add_pointer_t<WirelessDevice::OperationMode>>(_a[1]))); break;
        case 3: _t->wirelessCapabilitiesChanged((*reinterpret_cast<std::add_pointer_t<Capabilities>>(_a[1]))); break;
        case 4: _t->hardwareAddressChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->permanentHardwareAddressChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->wirelessPropertiesChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 7: _t->accessPointAppeared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->accessPointDisappeared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->networkAppeared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->networkDisappeared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->lastScanChanged((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WirelessDevice::*)(int )>(_a, &WirelessDevice::bitRateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WirelessDevice::*)(const QString & )>(_a, &WirelessDevice::activeAccessPointChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WirelessDevice::*)(WirelessDevice::OperationMode )>(_a, &WirelessDevice::modeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WirelessDevice::*)(Capabilities )>(_a, &WirelessDevice::wirelessCapabilitiesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WirelessDevice::*)(const QString & )>(_a, &WirelessDevice::hardwareAddressChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WirelessDevice::*)(const QString & )>(_a, &WirelessDevice::permanentHardwareAddressChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WirelessDevice::*)(uint )>(_a, &WirelessDevice::wirelessPropertiesChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WirelessDevice::*)(const QString & )>(_a, &WirelessDevice::accessPointAppeared, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WirelessDevice::*)(const QString & )>(_a, &WirelessDevice::accessPointDisappeared, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (WirelessDevice::*)(const QString & )>(_a, &WirelessDevice::networkAppeared, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (WirelessDevice::*)(const QString & )>(_a, &WirelessDevice::networkDisappeared, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (WirelessDevice::*)(const QDateTime & )>(_a, &WirelessDevice::lastScanChanged, 11))
            return;
    }
}

const QMetaObject *NetworkManager::WirelessDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NetworkManager::WirelessDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager14WirelessDeviceE_t>.strings))
        return static_cast<void*>(this);
    return Device::qt_metacast(_clname);
}

int NetworkManager::WirelessDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Device::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void NetworkManager::WirelessDevice::bitRateChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NetworkManager::WirelessDevice::activeAccessPointChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void NetworkManager::WirelessDevice::modeChanged(WirelessDevice::OperationMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void NetworkManager::WirelessDevice::wirelessCapabilitiesChanged(Capabilities _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void NetworkManager::WirelessDevice::hardwareAddressChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void NetworkManager::WirelessDevice::permanentHardwareAddressChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void NetworkManager::WirelessDevice::wirelessPropertiesChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void NetworkManager::WirelessDevice::accessPointAppeared(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void NetworkManager::WirelessDevice::accessPointDisappeared(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void NetworkManager::WirelessDevice::networkAppeared(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void NetworkManager::WirelessDevice::networkDisappeared(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void NetworkManager::WirelessDevice::lastScanChanged(const QDateTime & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}
QT_WARNING_POP
