/****************************************************************************
** Meta object code from reading C++ file 'gredevice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/gredevice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'gredevice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14NetworkManager9GreDeviceE_t {};
} // unnamed namespace

template <> constexpr inline auto NetworkManager::GreDevice::qt_create_metaobjectdata<qt_meta_tag_ZN14NetworkManager9GreDeviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NetworkManager::GreDevice",
        "inputFlagsChanged",
        "",
        "inputflags",
        "outputFlagsChanged",
        "outputFlags",
        "inputKeyChanged",
        "inputKey",
        "outputKeyChanged",
        "outputKey",
        "localEndChanged",
        "localEnd",
        "remoteEndChanged",
        "remoteEnd",
        "parentChanged",
        "parent",
        "pathMtuDiscoveryChanged",
        "pathMtuDiscovery",
        "tosChanged",
        "tos",
        "ttlChanged",
        "ttl",
        "inputFlags"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'inputFlagsChanged'
        QtMocHelpers::SignalData<void(ushort)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UShort, 3 },
        }}),
        // Signal 'outputFlagsChanged'
        QtMocHelpers::SignalData<void(ushort)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UShort, 5 },
        }}),
        // Signal 'inputKeyChanged'
        QtMocHelpers::SignalData<void(uint)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 7 },
        }}),
        // Signal 'outputKeyChanged'
        QtMocHelpers::SignalData<void(uint)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 9 },
        }}),
        // Signal 'localEndChanged'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'remoteEndChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'parentChanged'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Signal 'pathMtuDiscoveryChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Signal 'tosChanged'
        QtMocHelpers::SignalData<void(uchar)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 19 },
        }}),
        // Signal 'ttlChanged'
        QtMocHelpers::SignalData<void(uchar)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'inputFlags'
        QtMocHelpers::PropertyData<ushort>(22, QMetaType::UShort, QMC::DefaultPropertyFlags, 0),
        // property 'outputFlags'
        QtMocHelpers::PropertyData<ushort>(5, QMetaType::UShort, QMC::DefaultPropertyFlags, 1),
        // property 'inputKey'
        QtMocHelpers::PropertyData<uint>(7, QMetaType::UInt, QMC::DefaultPropertyFlags, 2),
        // property 'outputKey'
        QtMocHelpers::PropertyData<uint>(9, QMetaType::UInt, QMC::DefaultPropertyFlags, 3),
        // property 'localEnd'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags, 4),
        // property 'remoteEnd'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'parent'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags, 6),
        // property 'pathMtuDiscovery'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags, 7),
        // property 'tos'
        QtMocHelpers::PropertyData<uchar>(19, QMetaType::UChar, QMC::DefaultPropertyFlags, 8),
        // property 'ttl'
        QtMocHelpers::PropertyData<uchar>(21, QMetaType::UChar, QMC::DefaultPropertyFlags, 9),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GreDevice, qt_meta_tag_ZN14NetworkManager9GreDeviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NetworkManager::GreDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<Device::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager9GreDeviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager9GreDeviceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14NetworkManager9GreDeviceE_t>.metaTypes,
    nullptr
} };

void NetworkManager::GreDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GreDevice *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->inputFlagsChanged((*reinterpret_cast<std::add_pointer_t<ushort>>(_a[1]))); break;
        case 1: _t->outputFlagsChanged((*reinterpret_cast<std::add_pointer_t<ushort>>(_a[1]))); break;
        case 2: _t->inputKeyChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 3: _t->outputKeyChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 4: _t->localEndChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->remoteEndChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->parentChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->pathMtuDiscoveryChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->tosChanged((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1]))); break;
        case 9: _t->ttlChanged((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GreDevice::*)(ushort )>(_a, &GreDevice::inputFlagsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreDevice::*)(ushort )>(_a, &GreDevice::outputFlagsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreDevice::*)(uint )>(_a, &GreDevice::inputKeyChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreDevice::*)(uint )>(_a, &GreDevice::outputKeyChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreDevice::*)(const QString & )>(_a, &GreDevice::localEndChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreDevice::*)(const QString & )>(_a, &GreDevice::remoteEndChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreDevice::*)(const QString & )>(_a, &GreDevice::parentChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreDevice::*)(bool )>(_a, &GreDevice::pathMtuDiscoveryChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreDevice::*)(uchar )>(_a, &GreDevice::tosChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreDevice::*)(uchar )>(_a, &GreDevice::ttlChanged, 9))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<ushort*>(_v) = _t->inputFlags(); break;
        case 1: *reinterpret_cast<ushort*>(_v) = _t->outputFlags(); break;
        case 2: *reinterpret_cast<uint*>(_v) = _t->inputKey(); break;
        case 3: *reinterpret_cast<uint*>(_v) = _t->outputKey(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->localEnd(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->remoteEnd(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->parent(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->pathMtuDiscovery(); break;
        case 8: *reinterpret_cast<uchar*>(_v) = _t->tos(); break;
        case 9: *reinterpret_cast<uchar*>(_v) = _t->ttl(); break;
        default: break;
        }
    }
}

const QMetaObject *NetworkManager::GreDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NetworkManager::GreDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager9GreDeviceE_t>.strings))
        return static_cast<void*>(this);
    return Device::qt_metacast(_clname);
}

int NetworkManager::GreDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Device::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void NetworkManager::GreDevice::inputFlagsChanged(ushort _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NetworkManager::GreDevice::outputFlagsChanged(ushort _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void NetworkManager::GreDevice::inputKeyChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void NetworkManager::GreDevice::outputKeyChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void NetworkManager::GreDevice::localEndChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void NetworkManager::GreDevice::remoteEndChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void NetworkManager::GreDevice::parentChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void NetworkManager::GreDevice::pathMtuDiscoveryChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void NetworkManager::GreDevice::tosChanged(uchar _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void NetworkManager::GreDevice::ttlChanged(uchar _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
