/****************************************************************************
** Meta object code from reading C++ file 'iptunneldevice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/iptunneldevice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'iptunneldevice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14NetworkManager14IpTunnelDeviceE_t {};
} // unnamed namespace

template <> constexpr inline auto NetworkManager::IpTunnelDevice::qt_create_metaobjectdata<qt_meta_tag_ZN14NetworkManager14IpTunnelDeviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NetworkManager::IpTunnelDevice",
        "encapsulationLimitChanged",
        "",
        "limit",
        "flowLabelChanged",
        "flowLabel",
        "inputKeyChanged",
        "inputKey",
        "localChanged",
        "local",
        "modeChanged",
        "mode",
        "outputKeyChanged",
        "outputKey",
        "parentChanged",
        "parent",
        "pathMtuDiscoveryChanged",
        "pathMtuDiscovery",
        "remoteChanged",
        "remote",
        "tosChanged",
        "tos",
        "ttlChanged",
        "ttl",
        "encapsulationLimit",
        "NetworkManager::Device::Ptr"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'encapsulationLimitChanged'
        QtMocHelpers::SignalData<void(uchar)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 3 },
        }}),
        // Signal 'flowLabelChanged'
        QtMocHelpers::SignalData<void(uint)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 5 },
        }}),
        // Signal 'inputKeyChanged'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'localChanged'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'modeChanged'
        QtMocHelpers::SignalData<void(uint)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 11 },
        }}),
        // Signal 'outputKeyChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'parentChanged'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Signal 'pathMtuDiscoveryChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Signal 'remoteChanged'
        QtMocHelpers::SignalData<void(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Signal 'tosChanged'
        QtMocHelpers::SignalData<void(uchar)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 21 },
        }}),
        // Signal 'ttlChanged'
        QtMocHelpers::SignalData<void(uchar)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'encapsulationLimit'
        QtMocHelpers::PropertyData<uchar>(24, QMetaType::UChar, QMC::DefaultPropertyFlags, 0),
        // property 'flowLabel'
        QtMocHelpers::PropertyData<uint>(5, QMetaType::UInt, QMC::DefaultPropertyFlags, 1),
        // property 'inputKey'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'local'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'mode'
        QtMocHelpers::PropertyData<uint>(11, QMetaType::UInt, QMC::DefaultPropertyFlags, 4),
        // property 'outputKey'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'parent'
        QtMocHelpers::PropertyData<NetworkManager::Device::Ptr>(15, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 6),
        // property 'pathMtuDiscovery'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags, 7),
        // property 'remote'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags, 8),
        // property 'tos'
        QtMocHelpers::PropertyData<uchar>(21, QMetaType::UChar, QMC::DefaultPropertyFlags, 9),
        // property 'ttl'
        QtMocHelpers::PropertyData<uchar>(23, QMetaType::UChar, QMC::DefaultPropertyFlags, 10),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IpTunnelDevice, qt_meta_tag_ZN14NetworkManager14IpTunnelDeviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN14NetworkManager14IpTunnelDeviceE[] = {
    QMetaObject::SuperData::link<NetworkManager::Device::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject NetworkManager::IpTunnelDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<Device::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager14IpTunnelDeviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager14IpTunnelDeviceE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN14NetworkManager14IpTunnelDeviceE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14NetworkManager14IpTunnelDeviceE_t>.metaTypes,
    nullptr
} };

void NetworkManager::IpTunnelDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<IpTunnelDevice *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->encapsulationLimitChanged((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1]))); break;
        case 1: _t->flowLabelChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 2: _t->inputKeyChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->localChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->modeChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 5: _t->outputKeyChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->parentChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->pathMtuDiscoveryChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->remoteChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->tosChanged((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1]))); break;
        case 10: _t->ttlChanged((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (IpTunnelDevice::*)(uchar )>(_a, &IpTunnelDevice::encapsulationLimitChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (IpTunnelDevice::*)(uint )>(_a, &IpTunnelDevice::flowLabelChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (IpTunnelDevice::*)(const QString & )>(_a, &IpTunnelDevice::inputKeyChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (IpTunnelDevice::*)(const QString & )>(_a, &IpTunnelDevice::localChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (IpTunnelDevice::*)(uint )>(_a, &IpTunnelDevice::modeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (IpTunnelDevice::*)(const QString & )>(_a, &IpTunnelDevice::outputKeyChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (IpTunnelDevice::*)(const QString & )>(_a, &IpTunnelDevice::parentChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (IpTunnelDevice::*)(bool )>(_a, &IpTunnelDevice::pathMtuDiscoveryChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (IpTunnelDevice::*)(const QString & )>(_a, &IpTunnelDevice::remoteChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (IpTunnelDevice::*)(uchar )>(_a, &IpTunnelDevice::tosChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (IpTunnelDevice::*)(uchar )>(_a, &IpTunnelDevice::ttlChanged, 10))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uchar*>(_v) = _t->encapsulationLimit(); break;
        case 1: *reinterpret_cast<uint*>(_v) = _t->flowLabel(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->inputKey(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->local(); break;
        case 4: *reinterpret_cast<uint*>(_v) = _t->mode(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->outputKey(); break;
        case 6: *reinterpret_cast<NetworkManager::Device::Ptr*>(_v) = _t->parent(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->pathMtuDiscovery(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->remote(); break;
        case 9: *reinterpret_cast<uchar*>(_v) = _t->tos(); break;
        case 10: *reinterpret_cast<uchar*>(_v) = _t->ttl(); break;
        default: break;
        }
    }
}

const QMetaObject *NetworkManager::IpTunnelDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NetworkManager::IpTunnelDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager14IpTunnelDeviceE_t>.strings))
        return static_cast<void*>(this);
    return Device::qt_metacast(_clname);
}

int NetworkManager::IpTunnelDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Device::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void NetworkManager::IpTunnelDevice::encapsulationLimitChanged(uchar _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NetworkManager::IpTunnelDevice::flowLabelChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void NetworkManager::IpTunnelDevice::inputKeyChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void NetworkManager::IpTunnelDevice::localChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void NetworkManager::IpTunnelDevice::modeChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void NetworkManager::IpTunnelDevice::outputKeyChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void NetworkManager::IpTunnelDevice::parentChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void NetworkManager::IpTunnelDevice::pathMtuDiscoveryChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void NetworkManager::IpTunnelDevice::remoteChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void NetworkManager::IpTunnelDevice::tosChanged(uchar _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void NetworkManager::IpTunnelDevice::ttlChanged(uchar _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
