/****************************************************************************
** Meta object code from reading C++ file 'vpnplugin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/vpnplugin.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vpnplugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14NetworkManager9VpnPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto NetworkManager::VpnPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN14NetworkManager9VpnPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NetworkManager::VpnPlugin",
        "configChanged",
        "",
        "QVariantMap",
        "configuration",
        "failureChanged",
        "reason",
        "ip4ConfigChanged",
        "ip4config",
        "ip6ConfigChanged",
        "ip6config",
        "loginBannerChanged",
        "banner",
        "stateChanged",
        "VpnConnection::State",
        "state",
        "connect",
        "NMVariantMapMap",
        "connection",
        "disconnect",
        "needSecrets",
        "setConfig",
        "config",
        "setFailure",
        "setIp4Config",
        "setIp6Config",
        "onStateChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'configChanged'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'failureChanged'
        QtMocHelpers::SignalData<void(uint)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 6 },
        }}),
        // Signal 'ip4ConfigChanged'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 8 },
        }}),
        // Signal 'ip6ConfigChanged'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 10 },
        }}),
        // Signal 'loginBannerChanged'
        QtMocHelpers::SignalData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(VpnConnection::State)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'connect'
        QtMocHelpers::SlotData<void(const NMVariantMapMap &)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'disconnect'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'needSecrets'
        QtMocHelpers::SlotData<QString(const NMVariantMapMap &)>(20, 2, QMC::AccessProtected, QMetaType::QString, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'setConfig'
        QtMocHelpers::SlotData<void(const QVariantMap &)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 22 },
        }}),
        // Slot 'setFailure'
        QtMocHelpers::SlotData<void(const QString &)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'setIp4Config'
        QtMocHelpers::SlotData<void(const QVariantMap &)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 22 },
        }}),
        // Slot 'setIp6Config'
        QtMocHelpers::SlotData<void(const QVariantMap &)>(25, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 22 },
        }}),
        // Slot 'onStateChanged'
        QtMocHelpers::SlotData<void(uint)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::UInt, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VpnPlugin, qt_meta_tag_ZN14NetworkManager9VpnPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NetworkManager::VpnPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager9VpnPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager9VpnPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14NetworkManager9VpnPluginE_t>.metaTypes,
    nullptr
} };

void NetworkManager::VpnPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<VpnPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->configChanged((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 1: _t->failureChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 2: _t->ip4ConfigChanged((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 3: _t->ip6ConfigChanged((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 4: _t->loginBannerChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<VpnConnection::State>>(_a[1]))); break;
        case 6: _t->connect((*reinterpret_cast<std::add_pointer_t<NMVariantMapMap>>(_a[1]))); break;
        case 7: _t->disconnect(); break;
        case 8: { QString _r = _t->needSecrets((*reinterpret_cast<std::add_pointer_t<NMVariantMapMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->setConfig((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 10: _t->setFailure((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->setIp4Config((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 12: _t->setIp6Config((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 13: _t->onStateChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NMVariantMapMap >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NMVariantMapMap >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (VpnPlugin::*)(const QVariantMap & )>(_a, &VpnPlugin::configChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (VpnPlugin::*)(uint )>(_a, &VpnPlugin::failureChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (VpnPlugin::*)(const QVariantMap & )>(_a, &VpnPlugin::ip4ConfigChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (VpnPlugin::*)(const QVariantMap & )>(_a, &VpnPlugin::ip6ConfigChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (VpnPlugin::*)(const QString & )>(_a, &VpnPlugin::loginBannerChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (VpnPlugin::*)(VpnConnection::State )>(_a, &VpnPlugin::stateChanged, 5))
            return;
    }
}

const QMetaObject *NetworkManager::VpnPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NetworkManager::VpnPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager9VpnPluginE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int NetworkManager::VpnPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void NetworkManager::VpnPlugin::configChanged(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NetworkManager::VpnPlugin::failureChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void NetworkManager::VpnPlugin::ip4ConfigChanged(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void NetworkManager::VpnPlugin::ip6ConfigChanged(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void NetworkManager::VpnPlugin::loginBannerChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void NetworkManager::VpnPlugin::stateChanged(VpnConnection::State _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
