/****************************************************************************
** Meta object code from reading C++ file 'vpnplugininterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/dbus/vpnplugininterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vpnplugininterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN46OrgFreedesktopNetworkManagerVPNPluginInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgFreedesktopNetworkManagerVPNPluginInterface::qt_create_metaobjectdata<qt_meta_tag_ZN46OrgFreedesktopNetworkManagerVPNPluginInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgFreedesktopNetworkManagerVPNPluginInterface",
        "Config",
        "",
        "QVariant",
        "config",
        "Failure",
        "reason",
        "Ip4Config",
        "ip4config",
        "Ip6Config",
        "ip6config",
        "LoginBanner",
        "banner",
        "SecretsRequired",
        "message",
        "secrets",
        "StateChanged",
        "state",
        "Connect",
        "QDBusPendingReply<>",
        "NMVariantMapMap",
        "connection",
        "ConnectInteractive",
        "details",
        "Disconnect",
        "NeedSecrets",
        "QDBusPendingReply<QString>",
        "settings",
        "NewSecrets",
        "SetConfig",
        "SetFailure",
        "SetIp4Config",
        "SetIp6Config",
        "State"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Config'
        QtMocHelpers::SignalData<void(QVariant)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'Failure'
        QtMocHelpers::SignalData<void(uint)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 6 },
        }}),
        // Signal 'Ip4Config'
        QtMocHelpers::SignalData<void(QVariant)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 8 },
        }}),
        // Signal 'Ip6Config'
        QtMocHelpers::SignalData<void(QVariant)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 10 },
        }}),
        // Signal 'LoginBanner'
        QtMocHelpers::SignalData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Signal 'SecretsRequired'
        QtMocHelpers::SignalData<void(const QString &, const QStringList &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 }, { QMetaType::QStringList, 15 },
        }}),
        // Signal 'StateChanged'
        QtMocHelpers::SignalData<void(uint)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 17 },
        }}),
        // Slot 'Connect'
        QtMocHelpers::SlotData<QDBusPendingReply<>(NMVariantMapMap)>(18, 2, QMC::AccessPublic, 0x80000000 | 19, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'ConnectInteractive'
        QtMocHelpers::SlotData<QDBusPendingReply<>(NMVariantMapMap, QVariant)>(22, 2, QMC::AccessPublic, 0x80000000 | 19, {{
            { 0x80000000 | 20, 21 }, { 0x80000000 | 3, 23 },
        }}),
        // Slot 'Disconnect'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(24, 2, QMC::AccessPublic, 0x80000000 | 19),
        // Slot 'NeedSecrets'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(NMVariantMapMap)>(25, 2, QMC::AccessPublic, 0x80000000 | 26, {{
            { 0x80000000 | 20, 27 },
        }}),
        // Slot 'NewSecrets'
        QtMocHelpers::SlotData<QDBusPendingReply<>(NMVariantMapMap)>(28, 2, QMC::AccessPublic, 0x80000000 | 19, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'SetConfig'
        QtMocHelpers::SlotData<QDBusPendingReply<>(QVariant)>(29, 2, QMC::AccessPublic, 0x80000000 | 19, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'SetFailure'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(30, 2, QMC::AccessPublic, 0x80000000 | 19, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'SetIp4Config'
        QtMocHelpers::SlotData<QDBusPendingReply<>(QVariant)>(31, 2, QMC::AccessPublic, 0x80000000 | 19, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'SetIp6Config'
        QtMocHelpers::SlotData<QDBusPendingReply<>(QVariant)>(32, 2, QMC::AccessPublic, 0x80000000 | 19, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'State'
        QtMocHelpers::PropertyData<uint>(33, QMetaType::UInt, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopNetworkManagerVPNPluginInterface, qt_meta_tag_ZN46OrgFreedesktopNetworkManagerVPNPluginInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgFreedesktopNetworkManagerVPNPluginInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN46OrgFreedesktopNetworkManagerVPNPluginInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN46OrgFreedesktopNetworkManagerVPNPluginInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN46OrgFreedesktopNetworkManagerVPNPluginInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgFreedesktopNetworkManagerVPNPluginInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopNetworkManagerVPNPluginInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Config((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 1: _t->Failure((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 2: _t->Ip4Config((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 3: _t->Ip6Config((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 4: _t->LoginBanner((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->SecretsRequired((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 6: _t->StateChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 7: { QDBusPendingReply<> _r = _t->Connect((*reinterpret_cast<std::add_pointer_t<NMVariantMapMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<> _r = _t->ConnectInteractive((*reinterpret_cast<std::add_pointer_t<NMVariantMapMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<> _r = _t->Disconnect();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<QString> _r = _t->NeedSecrets((*reinterpret_cast<std::add_pointer_t<NMVariantMapMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<> _r = _t->NewSecrets((*reinterpret_cast<std::add_pointer_t<NMVariantMapMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<> _r = _t->SetConfig((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QDBusPendingReply<> _r = _t->SetFailure((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 14: { QDBusPendingReply<> _r = _t->SetIp4Config((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 15: { QDBusPendingReply<> _r = _t->SetIp6Config((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NMVariantMapMap >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NMVariantMapMap >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NMVariantMapMap >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NMVariantMapMap >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerVPNPluginInterface::*)(QVariant )>(_a, &OrgFreedesktopNetworkManagerVPNPluginInterface::Config, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerVPNPluginInterface::*)(uint )>(_a, &OrgFreedesktopNetworkManagerVPNPluginInterface::Failure, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerVPNPluginInterface::*)(QVariant )>(_a, &OrgFreedesktopNetworkManagerVPNPluginInterface::Ip4Config, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerVPNPluginInterface::*)(QVariant )>(_a, &OrgFreedesktopNetworkManagerVPNPluginInterface::Ip6Config, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerVPNPluginInterface::*)(const QString & )>(_a, &OrgFreedesktopNetworkManagerVPNPluginInterface::LoginBanner, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerVPNPluginInterface::*)(const QString & , const QStringList & )>(_a, &OrgFreedesktopNetworkManagerVPNPluginInterface::SecretsRequired, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerVPNPluginInterface::*)(uint )>(_a, &OrgFreedesktopNetworkManagerVPNPluginInterface::StateChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->state(); break;
        default: break;
        }
    }
}

const QMetaObject *OrgFreedesktopNetworkManagerVPNPluginInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgFreedesktopNetworkManagerVPNPluginInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN46OrgFreedesktopNetworkManagerVPNPluginInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgFreedesktopNetworkManagerVPNPluginInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void OrgFreedesktopNetworkManagerVPNPluginInterface::Config(QVariant _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OrgFreedesktopNetworkManagerVPNPluginInterface::Failure(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OrgFreedesktopNetworkManagerVPNPluginInterface::Ip4Config(QVariant _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OrgFreedesktopNetworkManagerVPNPluginInterface::Ip6Config(QVariant _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OrgFreedesktopNetworkManagerVPNPluginInterface::LoginBanner(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void OrgFreedesktopNetworkManagerVPNPluginInterface::SecretsRequired(const QString & _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void OrgFreedesktopNetworkManagerVPNPluginInterface::StateChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
