/****************************************************************************
** Meta object code from reading C++ file 'deviceinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/dbus/deviceinterface.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'deviceinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN43OrgFreedesktopNetworkManagerDeviceInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgFreedesktopNetworkManagerDeviceInterface::qt_create_metaobjectdata<qt_meta_tag_ZN43OrgFreedesktopNetworkManagerDeviceInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgFreedesktopNetworkManagerDeviceInterface",
        "StateChanged",
        "",
        "new_state",
        "old_state",
        "reason",
        "Delete",
        "QDBusPendingReply<>",
        "Disconnect",
        "GetAppliedConnection",
        "QDBusPendingReply<NMVariantMapMap,qulonglong>",
        "flags",
        "Reapply",
        "NMVariantMapMap",
        "connection",
        "version_id",
        "ActiveConnection",
        "QDBusObjectPath",
        "Autoconnect",
        "AvailableConnections",
        "QList<QDBusObjectPath>",
        "Capabilities",
        "DeviceType",
        "Dhcp4Config",
        "Dhcp6Config",
        "Driver",
        "DriverVersion",
        "FirmwareMissing",
        "FirmwareVersion",
        "Interface",
        "Ip4Address",
        "Ip4Config",
        "Ip6Config",
        "IpInterface",
        "InterfaceFlags",
        "LldpNeighbors",
        "NMVariantMapList",
        "Managed",
        "Metered",
        "Mtu",
        "NmPluginMissing",
        "PhysicalPortId",
        "Real",
        "State",
        "StateReason",
        "DeviceDBusStateReason",
        "Udi"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'StateChanged'
        QtMocHelpers::SignalData<void(uint, uint, uint)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 3 }, { QMetaType::UInt, 4 }, { QMetaType::UInt, 5 },
        }}),
        // Slot 'Delete'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(6, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'Disconnect'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(8, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'GetAppliedConnection'
        QtMocHelpers::SlotData<QDBusPendingReply<NMVariantMapMap,qulonglong>(uint)>(9, 2, QMC::AccessPublic, 0x80000000 | 10, {{
            { QMetaType::UInt, 11 },
        }}),
        // Slot 'Reapply'
        QtMocHelpers::SlotData<QDBusPendingReply<>(NMVariantMapMap, qulonglong, uint)>(12, 2, QMC::AccessPublic, 0x80000000 | 7, {{
            { 0x80000000 | 13, 14 }, { QMetaType::ULongLong, 15 }, { QMetaType::UInt, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'ActiveConnection'
        QtMocHelpers::PropertyData<QDBusObjectPath>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Autoconnect'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'AvailableConnections'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Capabilities'
        QtMocHelpers::PropertyData<uint>(21, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'DeviceType'
        QtMocHelpers::PropertyData<uint>(22, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'Dhcp4Config'
        QtMocHelpers::PropertyData<QDBusObjectPath>(23, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Dhcp6Config'
        QtMocHelpers::PropertyData<QDBusObjectPath>(24, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Driver'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'DriverVersion'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'FirmwareMissing'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'FirmwareVersion'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Interface'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Ip4Address'
        QtMocHelpers::PropertyData<uint>(30, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'Ip4Config'
        QtMocHelpers::PropertyData<QDBusObjectPath>(31, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Ip6Config'
        QtMocHelpers::PropertyData<QDBusObjectPath>(32, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'IpInterface'
        QtMocHelpers::PropertyData<QString>(33, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'InterfaceFlags'
        QtMocHelpers::PropertyData<uint>(34, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'LldpNeighbors'
        QtMocHelpers::PropertyData<NMVariantMapList>(35, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Managed'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'Metered'
        QtMocHelpers::PropertyData<uint>(38, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'Mtu'
        QtMocHelpers::PropertyData<uint>(39, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'NmPluginMissing'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'PhysicalPortId'
        QtMocHelpers::PropertyData<QString>(41, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Real'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'State'
        QtMocHelpers::PropertyData<uint>(43, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'StateReason'
        QtMocHelpers::PropertyData<DeviceDBusStateReason>(44, 0x80000000 | 45, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Udi'
        QtMocHelpers::PropertyData<QString>(46, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopNetworkManagerDeviceInterface, qt_meta_tag_ZN43OrgFreedesktopNetworkManagerDeviceInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgFreedesktopNetworkManagerDeviceInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN43OrgFreedesktopNetworkManagerDeviceInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN43OrgFreedesktopNetworkManagerDeviceInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN43OrgFreedesktopNetworkManagerDeviceInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgFreedesktopNetworkManagerDeviceInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopNetworkManagerDeviceInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->StateChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3]))); break;
        case 1: { QDBusPendingReply<> _r = _t->Delete();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<> _r = _t->Disconnect();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<NMVariantMapMap,qulonglong> _r = _t->GetAppliedConnection((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<NMVariantMapMap,qulonglong>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<> _r = _t->Reapply((*reinterpret_cast<std::add_pointer_t<NMVariantMapMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NMVariantMapMap >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerDeviceInterface::*)(uint , uint , uint )>(_a, &OrgFreedesktopNetworkManagerDeviceInterface::StateChanged, 0))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 25:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< DeviceDBusStateReason >(); break;
        case 17:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< NMVariantMapList >(); break;
        case 14:
        case 13:
        case 6:
        case 5:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QDBusObjectPath> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->activeConnection(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->autoconnect(); break;
        case 2: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->availableConnections(); break;
        case 3: *reinterpret_cast<uint*>(_v) = _t->capabilities(); break;
        case 4: *reinterpret_cast<uint*>(_v) = _t->deviceType(); break;
        case 5: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->dhcp4Config(); break;
        case 6: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->dhcp6Config(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->driver(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->driverVersion(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->firmwareMissing(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->firmwareVersion(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->interface(); break;
        case 12: *reinterpret_cast<uint*>(_v) = _t->ip4Address(); break;
        case 13: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->ip4Config(); break;
        case 14: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->ip6Config(); break;
        case 15: *reinterpret_cast<QString*>(_v) = _t->ipInterface(); break;
        case 16: *reinterpret_cast<uint*>(_v) = _t->interfaceFlags(); break;
        case 17: *reinterpret_cast<NMVariantMapList*>(_v) = _t->lldpNeighbors(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->managed(); break;
        case 19: *reinterpret_cast<uint*>(_v) = _t->metered(); break;
        case 20: *reinterpret_cast<uint*>(_v) = _t->mtu(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->nmPluginMissing(); break;
        case 22: *reinterpret_cast<QString*>(_v) = _t->physicalPortId(); break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->real(); break;
        case 24: *reinterpret_cast<uint*>(_v) = _t->state(); break;
        case 25: *reinterpret_cast<DeviceDBusStateReason*>(_v) = _t->stateReason(); break;
        case 26: *reinterpret_cast<QString*>(_v) = _t->udi(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setAutoconnect(*reinterpret_cast<bool*>(_v)); break;
        case 18: _t->setManaged(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OrgFreedesktopNetworkManagerDeviceInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgFreedesktopNetworkManagerDeviceInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN43OrgFreedesktopNetworkManagerDeviceInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgFreedesktopNetworkManagerDeviceInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    return _id;
}

// SIGNAL 0
void OrgFreedesktopNetworkManagerDeviceInterface::StateChanged(uint _t1, uint _t2, uint _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
