/****************************************************************************
** Meta object code from reading C++ file 'activeconnection.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/activeconnection.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'activeconnection.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14NetworkManager16ActiveConnectionE_t {};
} // unnamed namespace

template <> constexpr inline auto NetworkManager::ActiveConnection::qt_create_metaobjectdata<qt_meta_tag_ZN14NetworkManager16ActiveConnectionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NetworkManager::ActiveConnection",
        "connectionChanged",
        "",
        "NetworkManager::Connection::Ptr",
        "connection",
        "default4Changed",
        "isDefault",
        "default6Changed",
        "dhcp4ConfigChanged",
        "dhcp6ConfigChanged",
        "ipV4ConfigChanged",
        "ipV6ConfigChanged",
        "idChanged",
        "id",
        "typeChanged",
        "NetworkManager::ConnectionSettings::ConnectionType",
        "type",
        "masterChanged",
        "uni",
        "specificObjectChanged",
        "path",
        "stateChanged",
        "NetworkManager::ActiveConnection::State",
        "state",
        "stateChangedReason",
        "NetworkManager::ActiveConnection::Reason",
        "reason",
        "vpnChanged",
        "isVpn",
        "uuidChanged",
        "uuid",
        "devicesChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connectionChanged'
        QtMocHelpers::SignalData<void(const NetworkManager::Connection::Ptr &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'default4Changed'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Signal 'default6Changed'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Signal 'dhcp4ConfigChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dhcp6ConfigChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ipV4ConfigChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ipV6ConfigChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'idChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'typeChanged'
        QtMocHelpers::SignalData<void(NetworkManager::ConnectionSettings::ConnectionType)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'masterChanged'
        QtMocHelpers::SignalData<void(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Signal 'specificObjectChanged'
        QtMocHelpers::SignalData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(NetworkManager::ActiveConnection::State)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'stateChangedReason'
        QtMocHelpers::SignalData<void(NetworkManager::ActiveConnection::State, NetworkManager::ActiveConnection::Reason)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 }, { 0x80000000 | 25, 26 },
        }}),
        // Signal 'vpnChanged'
        QtMocHelpers::SignalData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Signal 'uuidChanged'
        QtMocHelpers::SignalData<void(const QString &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 },
        }}),
        // Signal 'devicesChanged'
        QtMocHelpers::SignalData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ActiveConnection, qt_meta_tag_ZN14NetworkManager16ActiveConnectionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NetworkManager::ActiveConnection::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager16ActiveConnectionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager16ActiveConnectionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14NetworkManager16ActiveConnectionE_t>.metaTypes,
    nullptr
} };

void NetworkManager::ActiveConnection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ActiveConnection *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connectionChanged((*reinterpret_cast<std::add_pointer_t<NetworkManager::Connection::Ptr>>(_a[1]))); break;
        case 1: _t->default4Changed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->default6Changed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->dhcp4ConfigChanged(); break;
        case 4: _t->dhcp6ConfigChanged(); break;
        case 5: _t->ipV4ConfigChanged(); break;
        case 6: _t->ipV6ConfigChanged(); break;
        case 7: _t->idChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->typeChanged((*reinterpret_cast<std::add_pointer_t<NetworkManager::ConnectionSettings::ConnectionType>>(_a[1]))); break;
        case 9: _t->masterChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->specificObjectChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<NetworkManager::ActiveConnection::State>>(_a[1]))); break;
        case 12: _t->stateChangedReason((*reinterpret_cast<std::add_pointer_t<NetworkManager::ActiveConnection::State>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<NetworkManager::ActiveConnection::Reason>>(_a[2]))); break;
        case 13: _t->vpnChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->uuidChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->devicesChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)(const NetworkManager::Connection::Ptr & )>(_a, &ActiveConnection::connectionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)(bool )>(_a, &ActiveConnection::default4Changed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)(bool )>(_a, &ActiveConnection::default6Changed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)()>(_a, &ActiveConnection::dhcp4ConfigChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)()>(_a, &ActiveConnection::dhcp6ConfigChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)()>(_a, &ActiveConnection::ipV4ConfigChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)()>(_a, &ActiveConnection::ipV6ConfigChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)(const QString & )>(_a, &ActiveConnection::idChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)(NetworkManager::ConnectionSettings::ConnectionType )>(_a, &ActiveConnection::typeChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)(const QString & )>(_a, &ActiveConnection::masterChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)(const QString & )>(_a, &ActiveConnection::specificObjectChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)(NetworkManager::ActiveConnection::State )>(_a, &ActiveConnection::stateChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)(NetworkManager::ActiveConnection::State , NetworkManager::ActiveConnection::Reason )>(_a, &ActiveConnection::stateChangedReason, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)(bool )>(_a, &ActiveConnection::vpnChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)(const QString & )>(_a, &ActiveConnection::uuidChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActiveConnection::*)()>(_a, &ActiveConnection::devicesChanged, 15))
            return;
    }
}

const QMetaObject *NetworkManager::ActiveConnection::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NetworkManager::ActiveConnection::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14NetworkManager16ActiveConnectionE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int NetworkManager::ActiveConnection::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void NetworkManager::ActiveConnection::connectionChanged(const NetworkManager::Connection::Ptr & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NetworkManager::ActiveConnection::default4Changed(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void NetworkManager::ActiveConnection::default6Changed(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void NetworkManager::ActiveConnection::dhcp4ConfigChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void NetworkManager::ActiveConnection::dhcp6ConfigChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void NetworkManager::ActiveConnection::ipV4ConfigChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void NetworkManager::ActiveConnection::ipV6ConfigChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void NetworkManager::ActiveConnection::idChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void NetworkManager::ActiveConnection::typeChanged(NetworkManager::ConnectionSettings::ConnectionType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void NetworkManager::ActiveConnection::masterChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void NetworkManager::ActiveConnection::specificObjectChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void NetworkManager::ActiveConnection::stateChanged(NetworkManager::ActiveConnection::State _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void NetworkManager::ActiveConnection::stateChangedReason(NetworkManager::ActiveConnection::State _t1, NetworkManager::ActiveConnection::Reason _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2);
}

// SIGNAL 13
void NetworkManager::ActiveConnection::vpnChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void NetworkManager::ActiveConnection::uuidChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void NetworkManager::ActiveConnection::devicesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}
QT_WARNING_POP
