/****************************************************************************
** Meta object code from reading C++ file 'activeconnectioninterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../networkmanager-qt-6.25.0/src/dbus/activeconnectioninterface.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'activeconnectioninterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN53OrgFreedesktopNetworkManagerConnectionActiveInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgFreedesktopNetworkManagerConnectionActiveInterface::qt_create_metaobjectdata<qt_meta_tag_ZN53OrgFreedesktopNetworkManagerConnectionActiveInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgFreedesktopNetworkManagerConnectionActiveInterface",
        "PropertiesChanged",
        "",
        "QVariantMap",
        "properties",
        "StateChanged",
        "state",
        "reason",
        "Connection",
        "QDBusObjectPath",
        "Default",
        "Default6",
        "Devices",
        "QList<QDBusObjectPath>",
        "Dhcp4Config",
        "Dhcp6Config",
        "Id",
        "Ip4Config",
        "Ip6Config",
        "Master",
        "SpecificObject",
        "State",
        "Type",
        "Uuid",
        "Vpn"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'PropertiesChanged'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'StateChanged'
        QtMocHelpers::SignalData<void(uint, uint)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 6 }, { QMetaType::UInt, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Connection'
        QtMocHelpers::PropertyData<QDBusObjectPath>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Default'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'Default6'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'Devices'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Dhcp4Config'
        QtMocHelpers::PropertyData<QDBusObjectPath>(14, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Dhcp6Config'
        QtMocHelpers::PropertyData<QDBusObjectPath>(15, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Id'
        QtMocHelpers::PropertyData<QString>(16, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Ip4Config'
        QtMocHelpers::PropertyData<QDBusObjectPath>(17, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Ip6Config'
        QtMocHelpers::PropertyData<QDBusObjectPath>(18, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Master'
        QtMocHelpers::PropertyData<QDBusObjectPath>(19, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'SpecificObject'
        QtMocHelpers::PropertyData<QDBusObjectPath>(20, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'State'
        QtMocHelpers::PropertyData<uint>(21, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'Type'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Uuid'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Vpn'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopNetworkManagerConnectionActiveInterface, qt_meta_tag_ZN53OrgFreedesktopNetworkManagerConnectionActiveInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgFreedesktopNetworkManagerConnectionActiveInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN53OrgFreedesktopNetworkManagerConnectionActiveInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN53OrgFreedesktopNetworkManagerConnectionActiveInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN53OrgFreedesktopNetworkManagerConnectionActiveInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgFreedesktopNetworkManagerConnectionActiveInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopNetworkManagerConnectionActiveInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->PropertiesChanged((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 1: _t->StateChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerConnectionActiveInterface::*)(const QVariantMap & )>(_a, &OrgFreedesktopNetworkManagerConnectionActiveInterface::PropertiesChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopNetworkManagerConnectionActiveInterface::*)(uint , uint )>(_a, &OrgFreedesktopNetworkManagerConnectionActiveInterface::StateChanged, 1))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
        case 9:
        case 8:
        case 7:
        case 5:
        case 4:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QDBusObjectPath> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->connection(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->default4(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->default6(); break;
        case 3: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->devices(); break;
        case 4: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->dhcp4Config(); break;
        case 5: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->dhcp6Config(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->id(); break;
        case 7: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->ip4Config(); break;
        case 8: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->ip6Config(); break;
        case 9: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->master(); break;
        case 10: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->specificObject(); break;
        case 11: *reinterpret_cast<uint*>(_v) = _t->state(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->type(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->uuid(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->vpn(); break;
        default: break;
        }
    }
}

const QMetaObject *OrgFreedesktopNetworkManagerConnectionActiveInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgFreedesktopNetworkManagerConnectionActiveInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN53OrgFreedesktopNetworkManagerConnectionActiveInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgFreedesktopNetworkManagerConnectionActiveInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void OrgFreedesktopNetworkManagerConnectionActiveInterface::PropertiesChanged(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OrgFreedesktopNetworkManagerConnectionActiveInterface::StateChanged(uint _t1, uint _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
